@@ -1,6 +1,7 @@
 """Schema module for GraphQL."""
 from collections import OrderedDict
 import logging
+import warnings
 
 from django.conf import settings
 from django.contrib.contenttypes.models import ContentType
@@ -165,8 +166,10 @@ def extend_schema_type_null_field_choice(schema_type, model):
 
         if hasattr(schema_type, resolver_name):
             logger.warning(
-                f"Unable to add {field.name} to {schema_type._meta.name} "
-                f"because there is already an attribute with the same name ({field_name})"
+                'Unable to add "%s" to %s because there is already an attribute mapped to the same name ("%s")',
+                field.name,
+                schema_type._meta.name,
+                field_name,
             )
             continue
 
@@ -225,19 +228,33 @@ def extend_schema_type_custom_field(schema_type, model):
         prefix = f"{settings.GRAPHQL_CUSTOM_FIELD_PREFIX}_"
 
     for field in cfs:
+        # 2.0 TODO: #824 replace field.name with field.slug
         field_name = f"{prefix}{str_to_var_name(field.name)}"
+        if str_to_var_name(field.name) != field.name:
+            # 2.0 TODO: str_to_var_name is lossy, it may cause different fields to map to the same field_name
+            # In 2.0 we should simply omit fields whose names/slugs are invalid in GraphQL, instead of mapping them.
+            warnings.warn(
+                f'Custom field "{field}" on {model._meta.verbose_name} does not have a GraphQL-safe name '
+                f'("{field.name}"); for now it will be mapped to the GraphQL name "{field_name}", '
+                "but in a future release this field may fail to appear in GraphQL.",
+                FutureWarning,
+            )
         resolver_name = f"resolve_{field_name}"
 
         if hasattr(schema_type, resolver_name):
             logger.warning(
-                f"Unable to add the custom field {field.name} to {schema_type._meta.name} "
-                f"because there is already an attribute with the same name ({field_name})"
+                'Unable to add the custom field "%s" to %s '
+                'because there is already an attribute mapped to the same name ("%s")',
+                field,
+                schema_type._meta.name,
+                field_name,
             )
             continue
 
         setattr(
             schema_type,
             resolver_name,
+            # 2.0 TODO: #824 field.slug
             generate_custom_field_resolver(field.name, resolver_name),
         )
 
@@ -268,13 +285,23 @@ def extend_schema_type_computed_field(schema_type, model):
 
     for field in cfs:
         field_name = f"{prefix}{str_to_var_name(field.slug)}"
+        if str_to_var_name(field.slug) != field.slug:
+            # 2.0 TODO: str_to_var_name is lossy, it may cause different fields to map to the same field_name
+            # In 2.0 we should simply omit fields whose slugs are invalid in GraphQL, instead of mapping them.
+            warnings.warn(
+                f'Computed field "{field}" on {model._meta.verbose_name} does not have a GraphQL-safe slug '
+                f'("{field.slug}"); for now it will be mapped to the GraphQL name "{field_name}", '
+                "but in a future release this field may fail to appear in GraphQL.",
+                FutureWarning,
+            )
         resolver_name = f"resolve_{field_name}"
 
         if hasattr(schema_type, resolver_name):
             logger.warning(
-                "Unable to add the computed field %s to %s because there is already an attribute with the same name (%s)",
-                field.slug,
-                schema_type._meta.slug,
+                'Unable to add the computed field "%s" to %s because '
+                'there is already an attribute mapped to the same name ("%s")',
+                field,
+                schema_type._meta.name,
                 field_name,
             )
             continue
@@ -362,14 +389,26 @@ def extend_schema_type_relationships(schema_type, model):
             if not relationship.symmetric and relationship.source_type == relationship.destination_type:
                 rel_name = f"{rel_name}_{peer_side}"
             resolver_name = f"resolve_{rel_name}"
+            if str_to_var_name(relationship.slug) != relationship.slug:
+                # 2.0 TODO: str_to_var_name is lossy, it may cause different relationships to map to the same rel_name
+                # In 2.0 we should simply omit relations whose slugs are invalid in GraphQL, instead of mapping them.
+                warnings.warn(
+                    f'Relationship "{relationship}" on {model._meta.verbose_name} does not have a GraphQL-safe slug '
+                    f'("{relationship.slug}"); for now it will be mapped to the GraphQL name "{rel_name}", '
+                    "but in a future release this relationship may fail to appear in GraphQL.",
+                    FutureWarning,
+                )
 
             if hasattr(schema_type, resolver_name):
                 # If a symmetric relationship, and this is destination side, we already added source side, expected
                 # Otherwise something is wrong and we should warn
                 if side != "destination" or not relationship.symmetric:
                     logger.warning(
-                        f"Unable to add the custom relationship {relationship.slug} to {schema_type._meta.name} "
-                        f"because there is already an attribute with the same name ({rel_name})"
+                        'Unable to add the custom relationship "%s" to %s '
+                        'because there is already an attribute mapped to the same name ("%s")',
+                        relationship,
+                        schema_type._meta.name,
+                        rel_name,
                     )
                 continue
 
@@ -380,13 +419,13 @@ def extend_schema_type_relationships(schema_type, model):
             if not peer_model:
                 # Could happen if, for example, we have a leftover relationship defined for a model from a plugin
                 # that is no longer installed/enabled
-                logger.warning(f"Unable to find peer model {peer_type} to create GraphQL relationship")
+                logger.warning("Unable to find peer model %s to create GraphQL relationship", peer_type)
                 continue
             type_identifier = f"{peer_model._meta.app_label}.{peer_model._meta.model_name}"
             rel_schema_type = registry["graphql_types"].get(type_identifier)
 
             if not rel_schema_type:
-                logger.warning(f"Unable to identify the GraphQL Object Type for {type_identifier} in the registry.")
+                logger.warning("Unable to identify the GraphQL Object Type for %s in the registry.", type_identifier)
                 continue
 
             if relationship.has_many(peer_side):
@@ -419,15 +458,19 @@ def generate_query_mixin():
 
         if single_item_name in class_attrs:
             logger.warning(
-                f"Unable to register the schema single type '{single_item_name}' in GraphQL, "
-                f"there is already another type {class_attrs[single_item_name]._type} registered under this name"
+                'Unable to register the schema single type "%s" in GraphQL, '
+                'as there is already another type "%s" registered under this name',
+                single_item_name,
+                class_attrs[single_item_name]._type,
             )
             return True
 
         if list_name in class_attrs:
             logger.warning(
-                f"Unable to register the schema list type '{list_name}' in GraphQL, "
-                f"there is already another type {class_attrs[list_name]._type} registered under this name"
+                'Unable to register the schema list type "%s" in GraphQL, '
+                'as there is already another type "%s" registered under this name',
+                list_name,
+                class_attrs[list_name]._type,
             )
             return True
 
@@ -445,8 +488,10 @@ def generate_query_mixin():
                 model = ct.model_class()
             except ContentType.DoesNotExist:
                 logger.warning(
-                    f"Unable to generate a schema type for the model '{app_name}.{model_name}' in GraphQL,"
-                    "this model doesn't have an associated ContentType, please create the Object manually."
+                    'Unable to generate a schema type for the model "%s.%s" in GraphQL, '
+                    "as this model doesn't have an associated ContentType. Please create the Object manually.",
+                    app_name,
+                    model_name,
                 )
                 continue
 
@@ -467,10 +512,11 @@ def generate_query_mixin():
 
         if type_identifier in registry["graphql_types"]:
             logger.warning(
-                f'Unable to load schema type for the model "{type_identifier}" as there is already another type '
+                'Unable to load schema type for the model "%s" as there is already another type '
                 "registered under this name. If you are seeing this message during plugin development, check to "
                 "make sure that you aren't using @extras_features(\"graphql\") on the same model you're also "
-                "defining a custom GraphQL type for."
+                "defining a custom GraphQL type for.",
+                type_identifier,
             )
         else:
             registry["graphql_types"][type_identifier] = schema_type
