@@ -8,7 +8,7 @@ from django.http import HttpResponseForbidden, HttpResponse
 from django.shortcuts import get_object_or_404
 from django.views.decorators.clickjacking import xframe_options_sameorigin
 from drf_spectacular.types import OpenApiTypes
-from drf_spectacular.utils import extend_schema, OpenApiParameter
+from drf_spectacular.utils import extend_schema, extend_schema_view, OpenApiParameter
 from rest_framework.decorators import action
 from rest_framework.mixins import ListModelMixin
 from rest_framework.permissions import IsAuthenticated
@@ -17,7 +17,6 @@ from rest_framework.routers import APIRootView
 from rest_framework.viewsets import GenericViewSet, ViewSet
 
 from nautobot.circuits.models import Circuit
-from nautobot.core.api.views import ModelViewSet
 from nautobot.core.api.exceptions import ServiceUnavailable
 from nautobot.dcim import filters
 from nautobot.dcim.models import (
@@ -36,6 +35,8 @@ from nautobot.dcim.models import (
     FrontPortTemplate,
     Interface,
     InterfaceTemplate,
+    Location,
+    LocationType,
     Manufacturer,
     InventoryItem,
     Platform,
@@ -57,14 +58,14 @@ from nautobot.dcim.models import (
 )
 from nautobot.extras.api.views import (
     ConfigContextQuerySetMixin,
-    CustomFieldModelViewSet,
+    NautobotModelViewSet,
     StatusViewSetMixin,
 )
 from nautobot.extras.choices import SecretsGroupAccessTypeChoices, SecretsGroupSecretTypeChoices
 from nautobot.extras.secrets.exceptions import SecretError
 from nautobot.ipam.models import Prefix, VLAN
 from nautobot.utilities.api import get_serializer_for_model
-from nautobot.utilities.utils import count_related
+from nautobot.utilities.utils import count_related, SerializerForAPIVersions, versioned_serializer_selector
 from nautobot.virtualization.models import VirtualMachine
 from . import serializers
 from .exceptions import MissingFilterException
@@ -134,7 +135,7 @@ class PassThroughPortMixin(object):
 #
 
 
-class RegionViewSet(CustomFieldModelViewSet):
+class RegionViewSet(NautobotModelViewSet):
     queryset = Region.objects.add_related_count(Region.objects.all(), Site, "region", "site_count", cumulative=True)
     serializer_class = serializers.RegionSerializer
     filterset_class = filters.RegionFilterSet
@@ -145,7 +146,7 @@ class RegionViewSet(CustomFieldModelViewSet):
 #
 
 
-class SiteViewSet(StatusViewSetMixin, CustomFieldModelViewSet):
+class SiteViewSet(StatusViewSetMixin, NautobotModelViewSet):
     queryset = Site.objects.prefetch_related("region", "status", "tenant", "tags").annotate(
         device_count=count_related(Device, "site"),
         rack_count=count_related(Rack, "site"),
@@ -158,12 +159,34 @@ class SiteViewSet(StatusViewSetMixin, CustomFieldModelViewSet):
     filterset_class = filters.SiteFilterSet
 
 
+#
+# Location types
+#
+
+
+class LocationTypeViewSet(NautobotModelViewSet):
+    queryset = LocationType.objects.prefetch_related("parent", "content_types")
+    serializer_class = serializers.LocationTypeSerializer
+    filterset_class = filters.LocationTypeFilterSet
+
+
+#
+# Locations
+#
+
+
+class LocationViewSet(StatusViewSetMixin, NautobotModelViewSet):
+    queryset = Location.objects.prefetch_related("location_type", "parent", "site", "status")
+    serializer_class = serializers.LocationSerializer
+    filterset_class = filters.LocationFilterSet
+
+
 #
 # Rack groups
 #
 
 
-class RackGroupViewSet(CustomFieldModelViewSet):
+class RackGroupViewSet(NautobotModelViewSet):
     queryset = RackGroup.objects.add_related_count(
         RackGroup.objects.all(), Rack, "group", "rack_count", cumulative=True
     ).prefetch_related("site")
@@ -176,7 +199,7 @@ class RackGroupViewSet(CustomFieldModelViewSet):
 #
 
 
-class RackRoleViewSet(CustomFieldModelViewSet):
+class RackRoleViewSet(NautobotModelViewSet):
     queryset = RackRole.objects.annotate(rack_count=count_related(Rack, "role"))
     serializer_class = serializers.RackRoleSerializer
     filterset_class = filters.RackRoleFilterSet
@@ -187,7 +210,7 @@ class RackRoleViewSet(CustomFieldModelViewSet):
 #
 
 
-class RackViewSet(StatusViewSetMixin, CustomFieldModelViewSet):
+class RackViewSet(StatusViewSetMixin, NautobotModelViewSet):
     queryset = Rack.objects.prefetch_related("site", "group__site", "status", "role", "tenant", "tags").annotate(
         device_count=count_related(Device, "rack"),
         powerfeed_count=count_related(PowerFeed, "rack"),
@@ -220,6 +243,7 @@ class RackViewSet(StatusViewSetMixin, CustomFieldModelViewSet):
                 legend_width=data["legend_width"],
                 include_images=data["include_images"],
                 base_url=request.build_absolute_uri("/"),
+                display_fullname=data["display_fullname"],
             )
             return HttpResponse(drawing.tostring(), content_type="image/svg+xml")
 
@@ -248,7 +272,7 @@ class RackViewSet(StatusViewSetMixin, CustomFieldModelViewSet):
 #
 
 
-class RackReservationViewSet(ModelViewSet):
+class RackReservationViewSet(NautobotModelViewSet):
     queryset = RackReservation.objects.prefetch_related("rack", "user", "tenant")
     serializer_class = serializers.RackReservationSerializer
     filterset_class = filters.RackReservationFilterSet
@@ -263,7 +287,7 @@ class RackReservationViewSet(ModelViewSet):
 #
 
 
-class ManufacturerViewSet(CustomFieldModelViewSet):
+class ManufacturerViewSet(NautobotModelViewSet):
     queryset = Manufacturer.objects.annotate(
         devicetype_count=count_related(DeviceType, "manufacturer"),
         inventoryitem_count=count_related(InventoryItem, "manufacturer"),
@@ -278,7 +302,7 @@ class ManufacturerViewSet(CustomFieldModelViewSet):
 #
 
 
-class DeviceTypeViewSet(CustomFieldModelViewSet):
+class DeviceTypeViewSet(NautobotModelViewSet):
     queryset = DeviceType.objects.prefetch_related("manufacturer", "tags").annotate(
         device_count=count_related(Device, "device_type")
     )
@@ -292,49 +316,49 @@ class DeviceTypeViewSet(CustomFieldModelViewSet):
 #
 
 
-class ConsolePortTemplateViewSet(CustomFieldModelViewSet):
+class ConsolePortTemplateViewSet(NautobotModelViewSet):
     queryset = ConsolePortTemplate.objects.prefetch_related("device_type__manufacturer")
     serializer_class = serializers.ConsolePortTemplateSerializer
     filterset_class = filters.ConsolePortTemplateFilterSet
 
 
-class ConsoleServerPortTemplateViewSet(CustomFieldModelViewSet):
+class ConsoleServerPortTemplateViewSet(NautobotModelViewSet):
     queryset = ConsoleServerPortTemplate.objects.prefetch_related("device_type__manufacturer")
     serializer_class = serializers.ConsoleServerPortTemplateSerializer
     filterset_class = filters.ConsoleServerPortTemplateFilterSet
 
 
-class PowerPortTemplateViewSet(CustomFieldModelViewSet):
+class PowerPortTemplateViewSet(NautobotModelViewSet):
     queryset = PowerPortTemplate.objects.prefetch_related("device_type__manufacturer")
     serializer_class = serializers.PowerPortTemplateSerializer
     filterset_class = filters.PowerPortTemplateFilterSet
 
 
-class PowerOutletTemplateViewSet(CustomFieldModelViewSet):
+class PowerOutletTemplateViewSet(NautobotModelViewSet):
     queryset = PowerOutletTemplate.objects.prefetch_related("device_type__manufacturer")
     serializer_class = serializers.PowerOutletTemplateSerializer
     filterset_class = filters.PowerOutletTemplateFilterSet
 
 
-class InterfaceTemplateViewSet(CustomFieldModelViewSet):
+class InterfaceTemplateViewSet(NautobotModelViewSet):
     queryset = InterfaceTemplate.objects.prefetch_related("device_type__manufacturer")
     serializer_class = serializers.InterfaceTemplateSerializer
     filterset_class = filters.InterfaceTemplateFilterSet
 
 
-class FrontPortTemplateViewSet(CustomFieldModelViewSet):
+class FrontPortTemplateViewSet(NautobotModelViewSet):
     queryset = FrontPortTemplate.objects.prefetch_related("device_type__manufacturer")
     serializer_class = serializers.FrontPortTemplateSerializer
     filterset_class = filters.FrontPortTemplateFilterSet
 
 
-class RearPortTemplateViewSet(CustomFieldModelViewSet):
+class RearPortTemplateViewSet(NautobotModelViewSet):
     queryset = RearPortTemplate.objects.prefetch_related("device_type__manufacturer")
     serializer_class = serializers.RearPortTemplateSerializer
     filterset_class = filters.RearPortTemplateFilterSet
 
 
-class DeviceBayTemplateViewSet(CustomFieldModelViewSet):
+class DeviceBayTemplateViewSet(NautobotModelViewSet):
     queryset = DeviceBayTemplate.objects.prefetch_related("device_type__manufacturer")
     serializer_class = serializers.DeviceBayTemplateSerializer
     filterset_class = filters.DeviceBayTemplateFilterSet
@@ -345,7 +369,7 @@ class DeviceBayTemplateViewSet(CustomFieldModelViewSet):
 #
 
 
-class DeviceRoleViewSet(CustomFieldModelViewSet):
+class DeviceRoleViewSet(NautobotModelViewSet):
     queryset = DeviceRole.objects.annotate(
         device_count=count_related(Device, "device_role"),
         virtualmachine_count=count_related(VirtualMachine, "role"),
@@ -359,7 +383,7 @@ class DeviceRoleViewSet(CustomFieldModelViewSet):
 #
 
 
-class PlatformViewSet(CustomFieldModelViewSet):
+class PlatformViewSet(NautobotModelViewSet):
     queryset = Platform.objects.annotate(
         device_count=count_related(Device, "platform"),
         virtualmachine_count=count_related(VirtualMachine, "platform"),
@@ -373,7 +397,7 @@ class PlatformViewSet(CustomFieldModelViewSet):
 #
 
 
-class DeviceViewSet(ConfigContextQuerySetMixin, StatusViewSetMixin, CustomFieldModelViewSet):
+class DeviceViewSet(ConfigContextQuerySetMixin, StatusViewSetMixin, NautobotModelViewSet):
     queryset = Device.objects.prefetch_related(
         "device_type__manufacturer",
         "device_role",
@@ -567,14 +591,14 @@ class DeviceViewSet(ConfigContextQuerySetMixin, StatusViewSetMixin, CustomFieldM
 #
 
 
-class ConsolePortViewSet(PathEndpointMixin, CustomFieldModelViewSet):
+class ConsolePortViewSet(PathEndpointMixin, NautobotModelViewSet):
     queryset = ConsolePort.objects.prefetch_related("device", "_path__destination", "cable", "_cable_peer", "tags")
     serializer_class = serializers.ConsolePortSerializer
     filterset_class = filters.ConsolePortFilterSet
     brief_prefetch_fields = ["device"]
 
 
-class ConsoleServerPortViewSet(PathEndpointMixin, CustomFieldModelViewSet):
+class ConsoleServerPortViewSet(PathEndpointMixin, NautobotModelViewSet):
     queryset = ConsoleServerPort.objects.prefetch_related(
         "device", "_path__destination", "cable", "_cable_peer", "tags"
     )
@@ -583,51 +607,83 @@ class ConsoleServerPortViewSet(PathEndpointMixin, CustomFieldModelViewSet):
     brief_prefetch_fields = ["device"]
 
 
-class PowerPortViewSet(PathEndpointMixin, CustomFieldModelViewSet):
+class PowerPortViewSet(PathEndpointMixin, NautobotModelViewSet):
     queryset = PowerPort.objects.prefetch_related("device", "_path__destination", "cable", "_cable_peer", "tags")
     serializer_class = serializers.PowerPortSerializer
     filterset_class = filters.PowerPortFilterSet
     brief_prefetch_fields = ["device"]
 
 
-class PowerOutletViewSet(PathEndpointMixin, CustomFieldModelViewSet):
+class PowerOutletViewSet(PathEndpointMixin, NautobotModelViewSet):
     queryset = PowerOutlet.objects.prefetch_related("device", "_path__destination", "cable", "_cable_peer", "tags")
     serializer_class = serializers.PowerOutletSerializer
     filterset_class = filters.PowerOutletFilterSet
     brief_prefetch_fields = ["device"]
 
 
-class InterfaceViewSet(PathEndpointMixin, CustomFieldModelViewSet):
+@extend_schema_view(
+    bulk_update=extend_schema(
+        responses={"200": serializers.InterfaceSerializerVersion12(many=True)}, versions=["1.2", "1.3"]
+    ),
+    bulk_partial_update=extend_schema(
+        responses={"200": serializers.InterfaceSerializerVersion12(many=True)}, versions=["1.2", "1.3"]
+    ),
+    create=extend_schema(responses={"201": serializers.InterfaceSerializerVersion12}, versions=["1.2", "1.3"]),
+    list=extend_schema(responses={"200": serializers.InterfaceSerializerVersion12(many=True)}, versions=["1.2", "1.3"]),
+    partial_update=extend_schema(responses={"200": serializers.InterfaceSerializerVersion12}, versions=["1.2", "1.3"]),
+    retrieve=extend_schema(responses={"200": serializers.InterfaceSerializerVersion12}, versions=["1.2", "1.3"]),
+    update=extend_schema(responses={"200": serializers.InterfaceSerializerVersion12}, versions=["1.2", "1.3"]),
+)
+class InterfaceViewSet(PathEndpointMixin, NautobotModelViewSet, StatusViewSetMixin):
     queryset = Interface.objects.prefetch_related(
-        "device", "_path__destination", "cable", "_cable_peer", "ip_addresses", "tags"
+        "device",
+        "parent_interface",
+        "bridge",
+        "lag",
+        "status",
+        "_path__destination",
+        "cable",
+        "_cable_peer",
+        "ip_addresses",
+        "tags",
     )
     serializer_class = serializers.InterfaceSerializer
     filterset_class = filters.InterfaceFilterSet
     brief_prefetch_fields = ["device"]
 
+    def get_serializer_class(self):
+        serializer_choices = (
+            SerializerForAPIVersions(versions=["1.2", "1.3"], serializer=serializers.InterfaceSerializerVersion12),
+        )
+        return versioned_serializer_selector(
+            obj=self,
+            serializer_choices=serializer_choices,
+            default_serializer=super().get_serializer_class(),
+        )
 
-class FrontPortViewSet(PassThroughPortMixin, CustomFieldModelViewSet):
+
+class FrontPortViewSet(PassThroughPortMixin, NautobotModelViewSet):
     queryset = FrontPort.objects.prefetch_related("device__device_type__manufacturer", "rear_port", "cable", "tags")
     serializer_class = serializers.FrontPortSerializer
     filterset_class = filters.FrontPortFilterSet
     brief_prefetch_fields = ["device"]
 
 
-class RearPortViewSet(PassThroughPortMixin, CustomFieldModelViewSet):
+class RearPortViewSet(PassThroughPortMixin, NautobotModelViewSet):
     queryset = RearPort.objects.prefetch_related("device__device_type__manufacturer", "cable", "tags")
     serializer_class = serializers.RearPortSerializer
     filterset_class = filters.RearPortFilterSet
     brief_prefetch_fields = ["device"]
 
 
-class DeviceBayViewSet(CustomFieldModelViewSet):
+class DeviceBayViewSet(NautobotModelViewSet):
     queryset = DeviceBay.objects.prefetch_related("installed_device").prefetch_related("tags")
     serializer_class = serializers.DeviceBaySerializer
     filterset_class = filters.DeviceBayFilterSet
     brief_prefetch_fields = ["device"]
 
 
-class InventoryItemViewSet(CustomFieldModelViewSet):
+class InventoryItemViewSet(NautobotModelViewSet):
     queryset = InventoryItem.objects.prefetch_related("device", "manufacturer").prefetch_related("tags")
     serializer_class = serializers.InventoryItemSerializer
     filterset_class = filters.InventoryItemFilterSet
@@ -666,7 +722,7 @@ class InterfaceConnectionViewSet(ListModelMixin, GenericViewSet):
 #
 
 
-class CableViewSet(StatusViewSetMixin, ModelViewSet):
+class CableViewSet(StatusViewSetMixin, NautobotModelViewSet):
     queryset = Cable.objects.prefetch_related("status", "termination_a", "termination_b")
     serializer_class = serializers.CableSerializer
     filterset_class = filters.CableFilterSet
@@ -677,7 +733,7 @@ class CableViewSet(StatusViewSetMixin, ModelViewSet):
 #
 
 
-class VirtualChassisViewSet(ModelViewSet):
+class VirtualChassisViewSet(NautobotModelViewSet):
     queryset = VirtualChassis.objects.prefetch_related("tags").annotate(
         member_count=count_related(Device, "virtual_chassis")
     )
@@ -691,7 +747,7 @@ class VirtualChassisViewSet(ModelViewSet):
 #
 
 
-class PowerPanelViewSet(ModelViewSet):
+class PowerPanelViewSet(NautobotModelViewSet):
     queryset = PowerPanel.objects.prefetch_related("site", "rack_group").annotate(
         powerfeed_count=count_related(PowerFeed, "power_panel")
     )
@@ -704,7 +760,7 @@ class PowerPanelViewSet(ModelViewSet):
 #
 
 
-class PowerFeedViewSet(PathEndpointMixin, StatusViewSetMixin, CustomFieldModelViewSet):
+class PowerFeedViewSet(PathEndpointMixin, StatusViewSetMixin, NautobotModelViewSet):
     queryset = PowerFeed.objects.prefetch_related(
         "power_panel",
         "rack",
