@@ -36,6 +36,7 @@ def update_custom_field_choice_data(field_id, old_value, new_value):
         # Loop through all field content types and search for values to update
         for ct in field.content_types.all():
             model = ct.model_class()
+            # 2.0 TODO: #824 field.slug rather than field.name
             for obj in model.objects.filter(**{f"_custom_field_data__{field.name}": old_value}):
                 obj._custom_field_data[field.name] = new_value
                 obj.save()
@@ -44,6 +45,7 @@ def update_custom_field_choice_data(field_id, old_value, new_value):
         # Loop through all field content types and search for values to update
         for ct in field.content_types.all():
             model = ct.model_class()
+            # 2.0 TODO: #824 field.slug rather than field.name
             for obj in model.objects.filter(**{f"_custom_field_data__{field.name}__contains": old_value}):
                 old_list = obj._custom_field_data[field.name]
                 new_list = [new_value if e == old_value else e for e in old_list]
@@ -55,6 +57,7 @@ def update_custom_field_choice_data(field_id, old_value, new_value):
         return False
 
 
+# 2.0 TODO: #824 rename field_name to field_slug
 @nautobot_task
 def delete_custom_field_data(field_name, content_type_pk_set):
     """
@@ -93,12 +96,13 @@ def provision_field(field_id, content_type_pk_set):
         for ct in ContentType.objects.filter(pk__in=content_type_pk_set):
             model = ct.model_class()
             for obj in model.objects.all():
-                obj._custom_field_data[field.name] = field.default
+                # 2.0 TODO: #824 field.slug rather than field.name
+                obj._custom_field_data.setdefault(field.name, field.default)
                 obj.save()
 
 
 @nautobot_task
-def process_webhook(webhook_pk, data, model_name, event, timestamp, username, request_id):
+def process_webhook(webhook_pk, data, model_name, event, timestamp, username, request_id, snapshots):
     """
     Make a POST request to the defined Webhook
     """
@@ -113,6 +117,7 @@ def process_webhook(webhook_pk, data, model_name, event, timestamp, username, re
         "username": username,
         "request_id": request_id,
         "data": data,
+        "snapshots": snapshots,
     }
 
     # Build the headers for the HTTP request
