@@ -139,9 +139,9 @@ class ConfigContextSchemaTestCase(SeleniumTestCase):
 
         # Assert Validation states
         self.assertEqual(
-            len(self.browser.find_by_xpath("//tbody/tr")), 3
+            len(self.browser.find_by_xpath("//div[@class[contains(., 'panel')]]//tbody/tr")), 3
         )  # 3 rows (config context, device, virtual machine)
-        for row in self.browser.find_by_xpath("//tbody/tr"):
+        for row in self.browser.find_by_xpath("//div[@class[contains(., 'panel')]]//tbody/tr"):
             self.assertEqual(
                 row.find_by_tag("td")[-2].html,
                 '<span class="text-success"><i class="mdi mdi-check-bold" title="Yes"></i></span>',
@@ -161,36 +161,38 @@ class ConfigContextSchemaTestCase(SeleniumTestCase):
 
         # Assert Validation states
         self.assertEqual(
-            len(self.browser.find_by_xpath("//tbody/tr")), 3
+            len(self.browser.find_by_xpath("//div[@class[contains(., 'panel')]]//tbody/tr")), 3
         )  # 3 rows (config context, device, virtual machine)
-        for row in self.browser.find_by_xpath("//tbody/tr"):
+        for row in self.browser.find_by_xpath("//div[@class[contains(., 'panel')]]//tbody/tr"):
             self.assertEqual(
                 row.find_by_tag("td")[-2].html,
                 '<span class="text-danger"><i class="mdi mdi-close-thick" title="No"></i></span><span class="text-danger">123 is not of type \'string\'</span>',
             )
 
         # Edit the device local context data and redirect back to the validation tab
-        self.browser.find_by_xpath("//tbody/tr")[1].find_by_tag("td")[-1].find_by_tag("a").click()
+        self.browser.find_by_xpath("//div[@class[contains(., 'panel')]]//tbody/tr")[1].find_by_tag("td")[
+            -1
+        ].find_by_tag("a").click()
         # Update the property "a" to be a string
         self.browser.fill("local_context_data", '{"a": "foo", "b": 456, "c": 777}')
         self.browser.find_by_text("Update").click()
 
         # Assert Validation states
         self.assertEqual(
-            len(self.browser.find_by_xpath("//tbody/tr")), 3
+            len(self.browser.find_by_xpath("//div[@class[contains(., 'panel')]]//tbody/tr")), 3
         )  # 3 rows (config context, device, virtual machine)
         # Config context still fails
         self.assertEqual(
-            self.browser.find_by_xpath("//tbody/tr")[0].find_by_tag("td")[-2].html,
+            self.browser.find_by_xpath("//div[@class[contains(., 'panel')]]//tbody/tr")[0].find_by_tag("td")[-2].html,
             '<span class="text-danger"><i class="mdi mdi-close-thick" title="No"></i></span><span class="text-danger">123 is not of type \'string\'</span>',
         )
         # Device now passes
         self.assertEqual(
-            self.browser.find_by_xpath("//tbody/tr")[1].find_by_tag("td")[-2].html,
+            self.browser.find_by_xpath("//div[@class[contains(., 'panel')]]//tbody/tr")[1].find_by_tag("td")[-2].html,
             '<span class="text-success"><i class="mdi mdi-check-bold" title="Yes"></i></span>',
         )
         # Virtual machine still fails
         self.assertEqual(
-            self.browser.find_by_xpath("//tbody/tr")[2].find_by_tag("td")[-2].html,
+            self.browser.find_by_xpath("//div[@class[contains(., 'panel')]]//tbody/tr")[2].find_by_tag("td")[-2].html,
             '<span class="text-danger"><i class="mdi mdi-close-thick" title="No"></i></span><span class="text-danger">123 is not of type \'string\'</span>',
         )
