@@ -1,26 +1,38 @@
+import datetime
 import json
-from io import StringIO
 import re
 import uuid
+from io import StringIO
+from unittest import mock
 
+from django.contrib.auth import get_user_model
 from django.contrib.contenttypes.models import ContentType
 from django.core.exceptions import ObjectDoesNotExist
 from django.core.files.uploadedfile import SimpleUploadedFile
 from django.core.management import call_command
 from django.core.management.base import CommandError
-from django.contrib.auth import get_user_model
 from django.test import override_settings
 from django.test.client import RequestFactory
+from django.utils import timezone
 
 from nautobot.dcim.models import DeviceRole, Site
-from nautobot.extras.choices import JobResultStatusChoices, LogLevelChoices
+from nautobot.extras.choices import (
+    JobExecutionType,
+    JobResultStatusChoices,
+    LogLevelChoices,
+    ObjectChangeEventContextChoices,
+)
+from nautobot.extras.context_managers import JobHookChangeContext, change_logging, web_request_context
 from nautobot.extras.jobs import get_job, run_job
-from nautobot.extras.models import FileProxy, Job, Status, CustomField, JobResult
+from nautobot.extras.models import CustomField, FileProxy, Job, JobHook, JobResult, ScheduledJob, Status
 from nautobot.extras.models.models import JobLogEntry
-from nautobot.utilities.testing import CeleryTestCase, TransactionTestCase, run_job_for_testing
-
-# Use the proper swappable User model
-User = get_user_model()
+from nautobot.utilities.testing import (
+    CeleryTestCase,
+    TestCase,
+    TransactionTestCase,
+    run_job_for_testing,
+)
+from nautobot.utilities.utils import get_changes_for_model
 
 
 def get_job_class_and_model(module, name):
@@ -57,7 +69,6 @@ class JobTest(TransactionTestCase):
         # Initialize fake request that will be required to execute Webhooks (in jobs.)
         self.request = RequestFactory().request(SERVER_NAME="WebRequestContext")
         self.request.id = uuid.uuid4()
-        self.user = User.objects.create_user(username="testuser")
         self.request.user = self.user
 
     def test_job_hard_time_limit_less_than_soft_time_limit(self):
@@ -87,6 +98,8 @@ class JobTest(TransactionTestCase):
         module = "test_pass"
         name = "TestPass"
         job_class, job_model = get_job_class_and_model(module, name)
+        job_model.enabled = True
+        job_model.validated_save()
         job_content_type = ContentType.objects.get(app_label="extras", model="job")
         job_result = JobResult.objects.create(
             name=job_model.class_path,
@@ -117,6 +130,29 @@ class JobTest(TransactionTestCase):
         job_result = create_job_result_and_run_job(module, name, commit=False)
         self.assertEqual(job_result.status, JobResultStatusChoices.STATUS_ERRORED)
 
+    def test_field_default(self):
+        """
+        Job test with field that is a default value that is falsey.
+        https://github.com/nautobot/nautobot/issues/2039
+        """
+        module = "test_field_default"
+        name = "TestFieldDefault"
+        job_class = get_job(f"local/{module}/{name}")
+        form = job_class().as_form()
+
+        self.assertHTMLEqual(
+            form.as_table(),
+            """<tr><th><label for="id_var_int">Var int:</label></th><td>
+<input class="form-control form-control" id="id_var_int" max="3600" name="var_int" placeholder="None" required type="number" value="0">
+<br><span class="helptext">Test default of 0 Falsey</span></td></tr>
+<tr><th><label for="id_var_int_no_default">Var int no default:</label></th><td>
+<input class="form-control form-control" id="id_var_int_no_default" max="3600" name="var_int_no_default" placeholder="None" type="number">
+<br><span class="helptext">Test default without default</span></td></tr>
+<tr><th><label for="id__commit">Commit changes:</label></th><td>
+<input checked id="id__commit" name="_commit" placeholder="Commit changes" type="checkbox">
+<br><span class="helptext">Commit changes to the database (uncheck for a dry-run)</span></td></tr>""",
+        )
+
     def test_field_order(self):
         """
         Job test with field order.
@@ -124,24 +160,8 @@ class JobTest(TransactionTestCase):
         module = "test_field_order"
         name = "TestFieldOrder"
         job_class = get_job(f"local/{module}/{name}")
-
         form = job_class().as_form()
-
-        self.assertHTMLEqual(
-            form.as_table(),
-            """<tr><th><label for="id_var1">Var1:</label></th><td>
-<input class="form-control form-control" id="id_var1" name="var1" placeholder="None" required type="file">
-<br><span class="helptext">Some file wants to be first</span></td></tr>
-<tr><th><label for="id_var2">Var2:</label></th><td>
-<input class="form-control form-control" id="id_var2" name="var2" placeholder="None" required type="text">
-<br><span class="helptext">Hello</span></td></tr>
-<tr><th><label for="id_var23">Var23:</label></th><td>
-<input class="form-control form-control" id="id_var23" name="var23" placeholder="None" required type="text">
-<br><span class="helptext">I want to be second</span></td></tr>
-<tr><th><label for="id__commit">Commit changes:</label></th><td>
-<input checked id="id__commit" name="_commit" placeholder="Commit changes" type="checkbox">
-<br><span class="helptext">Commit changes to the database (uncheck for a dry-run)</span></td></tr>""",
-        )
+        self.assertSequenceEqual(list(form.fields.keys()), ["var1", "var2", "var23", "_commit"])
 
     def test_no_field_order(self):
         """
@@ -150,21 +170,18 @@ class JobTest(TransactionTestCase):
         module = "test_no_field_order"
         name = "TestNoFieldOrder"
         job_class = get_job(f"local/{module}/{name}")
-
         form = job_class().as_form()
+        self.assertSequenceEqual(list(form.fields.keys()), ["var23", "var2", "_commit"])
 
-        self.assertHTMLEqual(
-            form.as_table(),
-            """<tr><th><label for="id_var23">Var23:</label></th><td>
-<input class="form-control form-control" id="id_var23" name="var23" placeholder="None" required type="text">
-<br><span class="helptext">I want to be second</span></td></tr>
-<tr><th><label for="id_var2">Var2:</label></th><td>
-<input class="form-control form-control" id="id_var2" name="var2" placeholder="None" required type="text">
-<br><span class="helptext">Hello</span></td></tr>
-<tr><th><label for="id__commit">Commit changes:</label></th><td>
-<input checked id="id__commit" name="_commit" placeholder="Commit changes" type="checkbox">
-<br><span class="helptext">Commit changes to the database (uncheck for a dry-run)</span></td></tr>""",
-        )
+    def test_no_field_order_inherited_variable(self):
+        """
+        Job test without field_order with a variable inherited from the base class
+        """
+        module = "test_no_field_order"
+        name = "TestDefaultFieldOrderWithInheritance"
+        job_class = get_job(f"local/{module}/{name}")
+        form = job_class().as_form()
+        self.assertSequenceEqual(list(form.fields.keys()), ["testvar1", "b_testvar2", "a_testvar3", "_commit"])
 
     def test_read_only_job_pass(self):
         """
@@ -346,6 +363,21 @@ class JobTest(TransactionTestCase):
         ).first()
         self.assertIn("Data should be a dictionary", log_failure.message)
 
+    def test_job_latest_result_property(self):
+        """
+        Job test to see if the latest_result property is indeed returning the most recent job result
+        """
+        module = "test_pass"
+        name = "TestPass"
+        job_result_1 = create_job_result_and_run_job(module, name, commit=False)
+        self.assertEqual(job_result_1.status, JobResultStatusChoices.STATUS_COMPLETED)
+        job_result_2 = create_job_result_and_run_job(module, name, commit=False)
+        self.assertEqual(job_result_2.status, JobResultStatusChoices.STATUS_COMPLETED)
+        job_class, job_model = get_job_class_and_model(module, name)
+        self.assertGreaterEqual(job_model.results.count(), 2)
+        latest_job_result = job_model.latest_result
+        self.assertEqual(job_result_2.completed, latest_job_result.completed)
+
 
 class JobFileUploadTest(TransactionTestCase):
     """Test a job that uploads/deletes files."""
@@ -361,7 +393,6 @@ class JobFileUploadTest(TransactionTestCase):
         # Initialize fake request that will be required to execute Webhooks (in jobs.)
         self.request = RequestFactory().request(SERVER_NAME="WebRequestContext")
         self.request.id = uuid.uuid4()
-        self.user = User.objects.create_user(username="testuser")
         self.request.user = self.user
 
     def test_run_job_pass(self):
@@ -534,11 +565,10 @@ class JobSiteCustomFieldTest(CeleryTestCase):
 
     def setUp(self):
         super().setUp()
-        user = User.objects.create(username="User1")
 
         self.request = RequestFactory().request(SERVER_NAME="WebRequestContext")
         self.request.id = uuid.uuid4()
-        self.request.user = user
+        self.request.user = self.user
 
     def test_run(self):
         self.clear_worker()
@@ -561,3 +591,143 @@ class JobSiteCustomFieldTest(CeleryTestCase):
         site_2 = Site.objects.filter(slug="test-site-two")
         self.assertEqual(site_2.count(), 1)
         self.assertEqual(site_2[0].cf["cf1"], "-")
+
+
+class JobHookReceiverTest(TransactionTestCase):
+    """
+    Test job hook receiver.
+    """
+
+    def setUp(self):
+        super().setUp()
+
+        # Initialize fake request that will be required to run jobs
+        self.request = RequestFactory().request(SERVER_NAME="WebRequestContext")
+        self.request.id = uuid.uuid4()
+        self.request.user = self.user
+
+        # generate an ObjectChange by creating a new site
+        with web_request_context(self.user):
+            site = Site(name="Test Site 1")
+            site.save()
+        site.refresh_from_db()
+        oc = get_changes_for_model(site).first()
+        self.data = {"object_change": oc.id}
+
+    def test_form_field(self):
+        module = "test_job_hook_receiver"
+        name = "TestJobHookReceiverLog"
+        job_class, job_model = get_job_class_and_model(module, name)
+        form = job_class().as_form()
+        self.assertCountEqual(form.fields.keys(), ["object_change", "_commit"])
+
+    def test_hidden(self):
+        module = "test_job_hook_receiver"
+        name = "TestJobHookReceiverLog"
+        job_class, job_model = get_job_class_and_model(module, name)
+        self.assertFalse(job_model.hidden)
+
+    def test_is_job_hook(self):
+
+        with self.subTest(expected=False):
+            module = "test_pass"
+            name = "TestPass"
+            job_class, job_model = get_job_class_and_model(module, name)
+            self.assertFalse(job_model.is_job_hook_receiver)
+
+        with self.subTest(expected=True):
+            module = "test_job_hook_receiver"
+            name = "TestJobHookReceiverLog"
+            job_class, job_model = get_job_class_and_model(module, name)
+            self.assertTrue(job_model.is_job_hook_receiver)
+
+    def test_object_change_context(self):
+        module = "test_job_hook_receiver"
+        name = "TestJobHookReceiverChange"
+        create_job_result_and_run_job(module, name, data=self.data, request=self.request)
+        test_site = Site.objects.get(name="test_jhr")
+        oc = get_changes_for_model(test_site).first()
+        self.assertEqual(oc.change_context, ObjectChangeEventContextChoices.CONTEXT_JOB_HOOK)
+        self.assertEqual(oc.user_id, self.user.pk)
+
+    def test_missing_receive_job_hook_method(self):
+        module = "test_job_hook_receiver"
+        name = "TestJobHookReceiverFail"
+        job_result = create_job_result_and_run_job(module, name, data=self.data, commit=False)
+        self.assertEqual(job_result.status, JobResultStatusChoices.STATUS_ERRORED)
+
+
+class JobHookTest(CeleryTestCase):
+    """
+    Test job hooks.
+    """
+
+    def setUp(self):
+        super().setUp()
+
+        module = "test_job_hook_receiver"
+        name = "TestJobHookReceiverLog"
+        self.job_class, self.job_model = get_job_class_and_model(module, name)
+        job_hook = JobHook(
+            name="JobHookTest",
+            type_create=True,
+            job=self.job_model,
+        )
+        obj_type = ContentType.objects.get_for_model(Site)
+        job_hook.save()
+        job_hook.content_types.set([obj_type])
+
+    def test_enqueue_job_hook(self):
+        self.clear_worker()
+        with web_request_context(user=self.user):
+            Site.objects.create(name="Test Job Hook Site 1")
+            self.wait_on_active_tasks()
+            job_result = JobResult.objects.get(job_model=self.job_model)
+            expected_log_messages = [
+                ("info", f"change: dcim | site Test Job Hook Site 1 created by {self.user.username}"),
+                ("info", "action: create"),
+                ("info", f"request.user: {self.user.username}"),
+                ("success", "Test Job Hook Site 1"),
+            ]
+            log_messages = JobLogEntry.objects.filter(job_result=job_result).values_list("log_level", "message")
+            self.assertSequenceEqual(log_messages, expected_log_messages)
+
+    @mock.patch.object(JobResult, "enqueue_job")
+    def test_enqueue_job_hook_skipped(self, mock):
+        change_context = JobHookChangeContext(user=self.user)
+        with change_logging(change_context):
+            Site.objects.create(name="Test Job Hook Site 2")
+
+        self.assertFalse(mock.called)
+
+
+class RemoveScheduledJobManagementCommandTestCase(TestCase):
+    def test_remove_stale_scheduled_jobs(self):
+        for i in range(1, 7):
+            ScheduledJob.objects.create(
+                name=f"test{i}",
+                task="nautobot.extras.jobs.scheduled_job_handler",
+                job_class="local/test_pass/TestPass",
+                interval=JobExecutionType.TYPE_FUTURE,
+                user=self.user,
+                start_time=timezone.now() - datetime.timedelta(days=i * 30),
+                one_off=i % 2 == 0,  # True / False
+            )
+
+        ScheduledJob.objects.create(
+            name="test7",
+            task="nautobot.extras.jobs.scheduled_job_handler",
+            job_class="local/test_pass/TestPass",
+            interval=JobExecutionType.TYPE_DAILY,
+            user=self.user,
+            start_time=timezone.now() - datetime.timedelta(days=180),
+        )
+
+        out = StringIO()
+        call_command("remove_stale_scheduled_jobs", 32, stdout=out)
+        self.assertEqual(ScheduledJob.objects.count(), 2)
+        self.assertIn("Stale scheduled jobs deleted successfully", out.getvalue())
+        self.assertTrue(ScheduledJob.objects.filter(name="test7").exists())
+        self.assertTrue(ScheduledJob.objects.filter(name="test1").exists())
+        for i in range(2, 7):
+            self.assertFalse(ScheduledJob.objects.filter(name=f"test{i}").exists())
