@@ -12,6 +12,9 @@ from nautobot.extras.models import (
     GitRepository,
     GraphQLQuery,
     Job,
+    Note,
+    JobHook,
+    Relationship,
     Secret,
     SecretsGroup,
     Status,
@@ -50,6 +53,12 @@ urlpatterns = [
         name="computedfield_changelog",
         kwargs={"model": ComputedField},
     ),
+    path(
+        "computed-fields/<slug:slug>/notes/",
+        views.ObjectNotesView.as_view(),
+        name="computedfield_notes",
+        kwargs={"model": ComputedField},
+    ),
     # Config contexts
     path(
         "config-contexts/",
@@ -92,6 +101,12 @@ urlpatterns = [
         name="configcontext_changelog",
         kwargs={"model": ConfigContext},
     ),
+    path(
+        "config-contexts/<uuid:pk>/notes/",
+        views.ObjectNotesView.as_view(),
+        name="configcontext_notes",
+        kwargs={"model": ConfigContext},
+    ),
     # Config context schema
     path(
         "config-context-schemas/",
@@ -139,6 +154,12 @@ urlpatterns = [
         name="configcontextschema_changelog",
         kwargs={"model": ConfigContextSchema},
     ),
+    path(
+        "config-context-schemas/<slug:slug>/notes/",
+        views.ObjectNotesView.as_view(),
+        name="configcontextschema_notes",
+        kwargs={"model": ConfigContextSchema},
+    ),
     # Custom fields
     path("custom-fields/", views.CustomFieldListView.as_view(), name="customfield_list"),
     path("custom-fields/add/", views.CustomFieldEditView.as_view(), name="customfield_add"),
@@ -147,24 +168,29 @@ urlpatterns = [
         views.CustomFieldBulkDeleteView.as_view(),
         name="customfield_bulk_delete",
     ),
-    # TODO: Migrate custom field model from name to slug #464
-    path("custom-fields/<str:name>/", views.CustomFieldView.as_view(), name="customfield"),
+    path("custom-fields/<slug:slug>/", views.CustomFieldView.as_view(), name="customfield"),
     path(
-        "custom-fields/<str:name>/edit/",
+        "custom-fields/<slug:slug>/edit/",
         views.CustomFieldEditView.as_view(),
         name="customfield_edit",
     ),
     path(
-        "custom-fields/<str:name>/delete/",
+        "custom-fields/<slug:slug>/delete/",
         views.CustomFieldDeleteView.as_view(),
         name="customfield_delete",
     ),
     path(
-        "custom-fields/<str:name>/changelog/",
+        "custom-fields/<slug:slug>/changelog/",
         views.ObjectChangeLogView.as_view(),
         name="customfield_changelog",
         kwargs={"model": CustomField},
     ),
+    path(
+        "custom-fields/<str:name>/notes/",
+        views.ObjectNotesView.as_view(),
+        name="customfield_notes",
+        kwargs={"model": CustomField},
+    ),
     # Custom links
     path("custom-links/", views.CustomLinkListView.as_view(), name="customlink_list"),
     path("custom-links/add/", views.CustomLinkEditView.as_view(), name="customlink_add"),
@@ -190,6 +216,12 @@ urlpatterns = [
         name="customlink_changelog",
         kwargs={"model": CustomLink},
     ),
+    path(
+        "custom-links/<uuid:pk>/notes/",
+        views.ObjectNotesView.as_view(),
+        name="customlink_notes",
+        kwargs={"model": CustomLink},
+    ),
     # Dynamic Groups
     path("dynamic-groups/", views.DynamicGroupListView.as_view(), name="dynamicgroup_list"),
     path("dynamic-groups/add/", views.DynamicGroupEditView.as_view(), name="dynamicgroup_add"),
@@ -207,6 +239,12 @@ urlpatterns = [
         name="dynamicgroup_changelog",
         kwargs={"model": DynamicGroup},
     ),
+    path(
+        "dynamic-groups/<str:slug>/notes/",
+        views.ObjectNotesView.as_view(),
+        name="dynamicgroup_notes",
+        kwargs={"model": DynamicGroup},
+    ),
     # Export Templates
     path(
         "export-templates/",
@@ -244,6 +282,12 @@ urlpatterns = [
         name="exporttemplate_changelog",
         kwargs={"model": ExportTemplate},
     ),
+    path(
+        "export-templates/<uuid:pk>/notes/",
+        views.ObjectNotesView.as_view(),
+        name="exporttemplate_notes",
+        kwargs={"model": ExportTemplate},
+    ),
     # Git repositories
     path(
         "git-repositories/",
@@ -291,6 +335,12 @@ urlpatterns = [
         name="gitrepository_changelog",
         kwargs={"model": GitRepository},
     ),
+    path(
+        "git-repositories/<str:slug>/notes/",
+        views.ObjectNotesView.as_view(),
+        name="gitrepository_notes",
+        kwargs={"model": GitRepository},
+    ),
     path(
         "git-repositories/<str:slug>/result/",
         views.GitRepositoryResultView.as_view(),
@@ -331,6 +381,12 @@ urlpatterns = [
         name="graphqlquery_changelog",
         kwargs={"model": GraphQLQuery},
     ),
+    path(
+        "graphql-queries/<uuid:pk>/notes/",
+        views.ObjectNotesView.as_view(),
+        name="graphqlquery_notes",
+        kwargs={"model": GraphQLQuery},
+    ),
     # Image attachments
     path(
         "image-attachments/<uuid:pk>/edit/",
@@ -381,9 +437,36 @@ urlpatterns = [
         name="job_changelog",
         kwargs={"model": Job},
     ),
+    path(
+        "jobs/<slug:slug>/notes/",
+        views.ObjectNotesView.as_view(),
+        name="job_notes",
+        kwargs={"model": Job},
+    ),
     # 2.0 TODO: JobView should actually be JobRunView, but keeping it as-is for backwards compatibility
     path("jobs/<slug:slug>/run/", views.JobView.as_view(), name="job_run"),
     path("jobs/<path:class_path>/", views.JobView.as_view(), name="job"),
+    # Job hooks
+    path("job-hooks/", views.JobHookListView.as_view(), name="jobhook_list"),
+    path("job-hooks/add/", views.JobHookEditView.as_view(), name="jobhook_add"),
+    path(
+        "job-hooks/delete/",
+        views.JobHookBulkDeleteView.as_view(),
+        name="jobhook_bulk_delete",
+    ),
+    path("job-hooks/<str:slug>/", views.JobHookView.as_view(), name="jobhook"),
+    path("job-hooks/<str:slug>/edit/", views.JobHookEditView.as_view(), name="jobhook_edit"),
+    path(
+        "job-hooks/<str:slug>/delete/",
+        views.JobHookDeleteView.as_view(),
+        name="jobhook_delete",
+    ),
+    path(
+        "job-hooks/<str:slug>/changelog/",
+        views.ObjectChangeLogView.as_view(),
+        name="jobhook_changelog",
+        kwargs={"model": JobHook},
+    ),
     # Generic job results
     path("job-results/", views.JobResultListView.as_view(), name="jobresult_list"),
     path("job-results/<uuid:pk>/", views.JobResultView.as_view(), name="jobresult"),
@@ -398,6 +481,17 @@ urlpatterns = [
         views.JobResultDeleteView.as_view(),
         name="jobresult_delete",
     ),
+    # Notes
+    path("notes/add/", views.NoteEditView.as_view(), name="note_add"),
+    path("notes/<slug:slug>/", views.NoteView.as_view(), name="note"),
+    path(
+        "notes/<slug:slug>/changelog/",
+        views.ObjectChangeLogView.as_view(),
+        name="note_changelog",
+        kwargs={"model": Note},
+    ),
+    path("notes/<slug:slug>/edit/", views.NoteEditView.as_view(), name="note_edit"),
+    path("notes/<slug:slug>/delete/", views.NoteDeleteView.as_view(), name="note_delete"),
     # Custom relationships
     path("relationships/", views.RelationshipListView.as_view(), name="relationship_list"),
     path(
@@ -411,27 +505,44 @@ urlpatterns = [
         name="relationship_bulk_delete",
     ),
     path(
-        "relationships/<uuid:pk>/edit/",
+        "relationships/<slug:slug>/",
+        views.RelationshipView.as_view(),
+        name="relationship",
+    ),
+    path(
+        "relationships/<slug:slug>/edit/",
         views.RelationshipEditView.as_view(),
         name="relationship_edit",
     ),
     path(
-        "relationships/<uuid:pk>/delete/",
+        "relationships/<slug:slug>/delete/",
         views.RelationshipDeleteView.as_view(),
         name="relationship_delete",
     ),
     path(
-        "relationships/associations/",
+        "relationships/<slug:slug>/changelog/",
+        views.ObjectChangeLogView.as_view(),
+        name="relationship_changelog",
+        kwargs={"model": Relationship},
+    ),
+    path(
+        "relationships/<slug:slug>/notes/",
+        views.ObjectNotesView.as_view(),
+        name="relationship_notes",
+        kwargs={"model": Relationship},
+    ),
+    path(
+        "relationship-associations/",
         views.RelationshipAssociationListView.as_view(),
         name="relationshipassociation_list",
     ),
     path(
-        "relationships/associations/delete/",
+        "relationship-associations/delete/",
         views.RelationshipAssociationBulkDeleteView.as_view(),
         name="relationshipassociation_bulk_delete",
     ),
     path(
-        "relationships/associations/<uuid:pk>/delete/",
+        "relationship-associations/<uuid:pk>/delete/",
         views.RelationshipAssociationDeleteView.as_view(),
         name="relationshipassociation_delete",
     ),
@@ -454,6 +565,12 @@ urlpatterns = [
         name="secret_changelog",
         kwargs={"model": Secret},
     ),
+    path(
+        "secrets/<str:slug>/notes/",
+        views.ObjectNotesView.as_view(),
+        name="secret_notes",
+        kwargs={"model": Secret},
+    ),
     path("secrets-groups/", views.SecretsGroupListView.as_view(), name="secretsgroup_list"),
     path("secrets-groups/add/", views.SecretsGroupEditView.as_view(), name="secretsgroup_add"),
     path("secrets-groups/delete/", views.SecretsGroupBulkDeleteView.as_view(), name="secretsgroup_bulk_delete"),
@@ -466,6 +583,12 @@ urlpatterns = [
         name="secretsgroup_changelog",
         kwargs={"model": SecretsGroup},
     ),
+    path(
+        "secrets-groups/<str:slug>/notes/",
+        views.ObjectNotesView.as_view(),
+        name="secretsgroup_notes",
+        kwargs={"model": SecretsGroup},
+    ),
     # Custom statuses
     path("statuses/", views.StatusListView.as_view(), name="status_list"),
     path("statuses/add/", views.StatusEditView.as_view(), name="status_add"),
@@ -489,6 +612,12 @@ urlpatterns = [
         name="status_changelog",
         kwargs={"model": Status},
     ),
+    path(
+        "statuses/<str:slug>/notes/",
+        views.ObjectNotesView.as_view(),
+        name="status_notes",
+        kwargs={"model": Status},
+    ),
     # Tags
     path("tags/", views.TagListView.as_view(), name="tag_list"),
     path("tags/add/", views.TagEditView.as_view(), name="tag_add"),
@@ -504,6 +633,12 @@ urlpatterns = [
         name="tag_changelog",
         kwargs={"model": Tag},
     ),
+    path(
+        "tags/<str:slug>/notes/",
+        views.ObjectNotesView.as_view(),
+        name="tag_notes",
+        kwargs={"model": Tag},
+    ),
     # Webhook
     path("webhooks/", views.WebhookListView.as_view(), name="webhook_list"),
     path("webhooks/add/", views.WebhookEditView.as_view(), name="webhook_add"),
@@ -525,4 +660,10 @@ urlpatterns = [
         name="webhook_changelog",
         kwargs={"model": Webhook},
     ),
+    path(
+        "webhooks/<uuid:pk>/notes/",
+        views.ObjectNotesView.as_view(),
+        name="webhook_notes",
+        kwargs={"model": Webhook},
+    ),
 ]
