@@ -1,18 +1,23 @@
 from django import forms
+from django.db.models import Q
 
+from nautobot.dcim.form_mixins import (
+    LocatableModelBulkEditFormMixin,
+    LocatableModelCSVFormMixin,
+    LocatableModelFilterFormMixin,
+    LocatableModelFormMixin,
+)
 from nautobot.dcim.models import Device, Interface, Rack, Region, Site
 from nautobot.extras.forms import (
-    AddRemoveTagsForm,
-    CustomFieldBulkEditForm,
-    CustomFieldFilterForm,
     CustomFieldModelCSVForm,
-    CustomFieldModelForm,
-    RelationshipModelForm,
-    StatusBulkEditFormMixin,
+    NautobotBulkEditForm,
+    NautobotModelForm,
+    NautobotFilterForm,
+    StatusModelBulkEditFormMixin,
     StatusModelCSVFormMixin,
-    StatusFilterFormMixin,
+    StatusModelFilterFormMixin,
+    TagsBulkEditFormMixin,
 )
-from nautobot.extras.models import Tag
 from nautobot.tenancy.forms import TenancyFilterForm, TenancyForm
 from nautobot.tenancy.models import Tenant
 from nautobot.utilities.forms import (
@@ -70,10 +75,9 @@ IPADDRESS_MASK_LENGTH_CHOICES = add_blank_choice(
 #
 
 
-class VRFForm(BootstrapMixin, TenancyForm, CustomFieldModelForm, RelationshipModelForm):
+class VRFForm(NautobotModelForm, TenancyForm):
     import_targets = DynamicModelMultipleChoiceField(queryset=RouteTarget.objects.all(), required=False)
     export_targets = DynamicModelMultipleChoiceField(queryset=RouteTarget.objects.all(), required=False)
-    tags = DynamicModelMultipleChoiceField(queryset=Tag.objects.all(), required=False)
 
     class Meta:
         model = VRF
@@ -109,7 +113,7 @@ class VRFCSVForm(CustomFieldModelCSVForm):
         fields = VRF.csv_headers
 
 
-class VRFBulkEditForm(BootstrapMixin, AddRemoveTagsForm, CustomFieldBulkEditForm):
+class VRFBulkEditForm(TagsBulkEditFormMixin, NautobotBulkEditForm):
     pk = forms.ModelMultipleChoiceField(queryset=VRF.objects.all(), widget=forms.MultipleHiddenInput())
     tenant = DynamicModelChoiceField(queryset=Tenant.objects.all(), required=False)
     enforce_unique = forms.NullBooleanField(
@@ -124,7 +128,7 @@ class VRFBulkEditForm(BootstrapMixin, AddRemoveTagsForm, CustomFieldBulkEditForm
         ]
 
 
-class VRFFilterForm(BootstrapMixin, TenancyFilterForm, CustomFieldFilterForm):
+class VRFFilterForm(NautobotFilterForm, TenancyFilterForm):
     model = VRF
     field_order = ["q", "import_target", "export_target", "tenant_group", "tenant"]
     q = forms.CharField(required=False, label="Search")
@@ -142,9 +146,7 @@ class VRFFilterForm(BootstrapMixin, TenancyFilterForm, CustomFieldFilterForm):
 #
 
 
-class RouteTargetForm(BootstrapMixin, TenancyForm, CustomFieldModelForm, RelationshipModelForm):
-    tags = DynamicModelMultipleChoiceField(queryset=Tag.objects.all(), required=False)
-
+class RouteTargetForm(NautobotModelForm, TenancyForm):
     class Meta:
         model = RouteTarget
         fields = [
@@ -169,7 +171,7 @@ class RouteTargetCSVForm(CustomFieldModelCSVForm):
         fields = RouteTarget.csv_headers
 
 
-class RouteTargetBulkEditForm(BootstrapMixin, AddRemoveTagsForm, CustomFieldBulkEditForm):
+class RouteTargetBulkEditForm(TagsBulkEditFormMixin, NautobotBulkEditForm):
     pk = forms.ModelMultipleChoiceField(queryset=RouteTarget.objects.all(), widget=forms.MultipleHiddenInput())
     tenant = DynamicModelChoiceField(queryset=Tenant.objects.all(), required=False)
     description = forms.CharField(max_length=200, required=False)
@@ -181,7 +183,7 @@ class RouteTargetBulkEditForm(BootstrapMixin, AddRemoveTagsForm, CustomFieldBulk
         ]
 
 
-class RouteTargetFilterForm(BootstrapMixin, TenancyFilterForm, CustomFieldFilterForm):
+class RouteTargetFilterForm(NautobotFilterForm, TenancyFilterForm):
     model = RouteTarget
     field_order = [
         "q",
@@ -206,7 +208,7 @@ class RouteTargetFilterForm(BootstrapMixin, TenancyFilterForm, CustomFieldFilter
 #
 
 
-class RIRForm(BootstrapMixin, CustomFieldModelForm, RelationshipModelForm):
+class RIRForm(NautobotModelForm):
     slug = SlugField()
 
     class Meta:
@@ -228,7 +230,7 @@ class RIRCSVForm(CustomFieldModelCSVForm):
         }
 
 
-class RIRFilterForm(BootstrapMixin, CustomFieldFilterForm):
+class RIRFilterForm(NautobotFilterForm):
     model = RIR
     is_private = forms.NullBooleanField(
         required=False,
@@ -242,9 +244,8 @@ class RIRFilterForm(BootstrapMixin, CustomFieldFilterForm):
 #
 
 
-class AggregateForm(BootstrapMixin, TenancyForm, PrefixFieldMixin, CustomFieldModelForm, RelationshipModelForm):
+class AggregateForm(NautobotModelForm, TenancyForm, PrefixFieldMixin):
     rir = DynamicModelChoiceField(queryset=RIR.objects.all(), label="RIR")
-    tags = DynamicModelMultipleChoiceField(queryset=Tag.objects.all(), required=False)
 
     class Meta:
         model = Aggregate
@@ -280,7 +281,7 @@ class AggregateCSVForm(PrefixFieldMixin, CustomFieldModelCSVForm):
         fields = Aggregate.csv_headers
 
 
-class AggregateBulkEditForm(BootstrapMixin, AddRemoveTagsForm, CustomFieldBulkEditForm):
+class AggregateBulkEditForm(TagsBulkEditFormMixin, NautobotBulkEditForm):
     pk = forms.ModelMultipleChoiceField(queryset=Aggregate.objects.all(), widget=forms.MultipleHiddenInput())
     rir = DynamicModelChoiceField(queryset=RIR.objects.all(), required=False, label="RIR")
     tenant = DynamicModelChoiceField(queryset=Tenant.objects.all(), required=False)
@@ -297,7 +298,7 @@ class AggregateBulkEditForm(BootstrapMixin, AddRemoveTagsForm, CustomFieldBulkEd
         }
 
 
-class AggregateFilterForm(BootstrapMixin, TenancyFilterForm, CustomFieldFilterForm):
+class AggregateFilterForm(NautobotFilterForm, TenancyFilterForm):
     model = Aggregate
     field_order = [
         "q",
@@ -320,7 +321,7 @@ class AggregateFilterForm(BootstrapMixin, TenancyFilterForm, CustomFieldFilterFo
 #
 
 
-class RoleForm(BootstrapMixin, CustomFieldModelForm, RelationshipModelForm):
+class RoleForm(NautobotModelForm):
     slug = SlugField()
 
     class Meta:
@@ -344,19 +345,12 @@ class RoleCSVForm(CustomFieldModelCSVForm):
 #
 
 
-class PrefixForm(BootstrapMixin, PrefixFieldMixin, TenancyForm, CustomFieldModelForm, RelationshipModelForm):
+class PrefixForm(LocatableModelFormMixin, NautobotModelForm, TenancyForm, PrefixFieldMixin):
     vrf = DynamicModelChoiceField(
         queryset=VRF.objects.all(),
         required=False,
         label="VRF",
     )
-    region = DynamicModelChoiceField(queryset=Region.objects.all(), required=False, initial_params={"sites": "$site"})
-    site = DynamicModelChoiceField(
-        queryset=Site.objects.all(),
-        required=False,
-        null_option="None",
-        query_params={"region_id": "$region"},
-    )
     vlan_group = DynamicModelChoiceField(
         queryset=VLANGroup.objects.all(),
         required=False,
@@ -375,7 +369,6 @@ class PrefixForm(BootstrapMixin, PrefixFieldMixin, TenancyForm, CustomFieldModel
         },
     )
     role = DynamicModelChoiceField(queryset=Role.objects.all(), required=False)
-    tags = DynamicModelMultipleChoiceField(queryset=Tag.objects.all(), required=False)
 
     class Meta:
         model = Prefix
@@ -383,6 +376,7 @@ class PrefixForm(BootstrapMixin, PrefixFieldMixin, TenancyForm, CustomFieldModel
             "prefix",
             "vrf",
             "site",
+            "location",
             "vlan",
             "status",
             "role",
@@ -400,7 +394,7 @@ class PrefixForm(BootstrapMixin, PrefixFieldMixin, TenancyForm, CustomFieldModel
         self.fields["vrf"].empty_label = "Global"
 
 
-class PrefixCSVForm(PrefixFieldMixin, StatusModelCSVFormMixin, CustomFieldModelCSVForm):
+class PrefixCSVForm(PrefixFieldMixin, LocatableModelCSVFormMixin, StatusModelCSVFormMixin, CustomFieldModelCSVForm):
     vrf = CSVModelChoiceField(
         queryset=VRF.objects.all(),
         to_field_name="name",
@@ -413,12 +407,6 @@ class PrefixCSVForm(PrefixFieldMixin, StatusModelCSVFormMixin, CustomFieldModelC
         to_field_name="name",
         help_text="Assigned tenant",
     )
-    site = CSVModelChoiceField(
-        queryset=Site.objects.all(),
-        required=False,
-        to_field_name="name",
-        help_text="Assigned site",
-    )
     vlan_group = CSVModelChoiceField(
         queryset=VLANGroup.objects.all(),
         required=False,
@@ -455,10 +443,13 @@ class PrefixCSVForm(PrefixFieldMixin, StatusModelCSVFormMixin, CustomFieldModelC
             self.fields["vlan"].queryset = self.fields["vlan"].queryset.filter(**params)
 
 
-class PrefixBulkEditForm(BootstrapMixin, AddRemoveTagsForm, StatusBulkEditFormMixin, CustomFieldBulkEditForm):
+class PrefixBulkEditForm(
+    TagsBulkEditFormMixin,
+    LocatableModelBulkEditFormMixin,
+    StatusModelBulkEditFormMixin,
+    NautobotBulkEditForm,
+):
     pk = forms.ModelMultipleChoiceField(queryset=Prefix.objects.all(), widget=forms.MultipleHiddenInput())
-    region = DynamicModelChoiceField(queryset=Region.objects.all(), required=False, to_field_name="slug")
-    site = DynamicModelChoiceField(queryset=Site.objects.all(), required=False, query_params={"region": "$region"})
     vrf = DynamicModelChoiceField(
         queryset=VRF.objects.all(),
         required=False,
@@ -471,8 +462,10 @@ class PrefixBulkEditForm(BootstrapMixin, AddRemoveTagsForm, StatusBulkEditFormMi
     description = forms.CharField(max_length=100, required=False)
 
     class Meta:
+        model = Prefix
         nullable_fields = [
             "site",
+            "location",
             "vrf",
             "tenant",
             "role",
@@ -480,7 +473,12 @@ class PrefixBulkEditForm(BootstrapMixin, AddRemoveTagsForm, StatusBulkEditFormMi
         ]
 
 
-class PrefixFilterForm(BootstrapMixin, TenancyFilterForm, StatusFilterFormMixin, CustomFieldFilterForm):
+class PrefixFilterForm(
+    NautobotFilterForm,
+    LocatableModelFilterFormMixin,
+    TenancyFilterForm,
+    StatusModelFilterFormMixin,
+):
     model = Prefix
     field_order = [
         "q",
@@ -492,13 +490,14 @@ class PrefixFilterForm(BootstrapMixin, TenancyFilterForm, StatusFilterFormMixin,
         "status",
         "region",
         "site",
+        "location",
         "role",
         "tenant_group",
         "tenant",
         "is_pool",
         "expand",
     ]
-    mask_length__lte = forms.IntegerField(widget=forms.HiddenInput())
+    mask_length__lte = forms.IntegerField(widget=forms.HiddenInput(), required=False)
     q = forms.CharField(required=False, label="Search")
     within_include = forms.CharField(
         required=False,
@@ -528,14 +527,6 @@ class PrefixFilterForm(BootstrapMixin, TenancyFilterForm, StatusFilterFormMixin,
         null_option="Global",
     )
     present_in_vrf_id = DynamicModelChoiceField(queryset=VRF.objects.all(), required=False, label="Present in VRF")
-    region = DynamicModelMultipleChoiceField(queryset=Region.objects.all(), to_field_name="slug", required=False)
-    site = DynamicModelMultipleChoiceField(
-        queryset=Site.objects.all(),
-        to_field_name="slug",
-        required=False,
-        null_option="None",
-        query_params={"region": "$region"},
-    )
     role = DynamicModelMultipleChoiceField(
         queryset=Role.objects.all(),
         to_field_name="slug",
@@ -555,14 +546,7 @@ class PrefixFilterForm(BootstrapMixin, TenancyFilterForm, StatusFilterFormMixin,
 #
 
 
-class IPAddressForm(
-    BootstrapMixin,
-    TenancyForm,
-    ReturnURLForm,
-    AddressFieldMixin,
-    CustomFieldModelForm,
-    RelationshipModelForm,
-):
+class IPAddressForm(NautobotModelForm, TenancyForm, ReturnURLForm, AddressFieldMixin):
     device = DynamicModelChoiceField(
         queryset=Device.objects.all(),
         required=False,
@@ -642,7 +626,6 @@ class IPAddressForm(
         },
     )
     primary_for_parent = forms.BooleanField(required=False, label="Make this the primary IP for the device/VM")
-    tags = DynamicModelMultipleChoiceField(queryset=Tag.objects.all(), required=False)
 
     class Meta:
         model = IPAddress
@@ -676,18 +659,18 @@ class IPAddressForm(
         initial = kwargs.get("initial", {}).copy()
 
         if instance:
-            if type(instance.assigned_object) is Interface:
+            if isinstance(instance.assigned_object, Interface):
                 initial["interface"] = instance.assigned_object
-            elif type(instance.assigned_object) is VMInterface:
+            elif isinstance(instance.assigned_object, VMInterface):
                 initial["vminterface"] = instance.assigned_object
             if instance.nat_inside:
                 nat_inside_parent = instance.nat_inside.assigned_object
-                if type(nat_inside_parent) is Interface:
+                if isinstance(nat_inside_parent, Interface):
                     initial["nat_site"] = nat_inside_parent.device.site.pk
                     if nat_inside_parent.device.rack:
                         initial["nat_rack"] = nat_inside_parent.device.rack.pk
                     initial["nat_device"] = nat_inside_parent.device.pk
-                elif type(nat_inside_parent) is VMInterface:
+                elif isinstance(nat_inside_parent, VMInterface):
                     initial["nat_cluster"] = nat_inside_parent.virtual_machine.cluster.pk
                     initial["nat_virtual_machine"] = nat_inside_parent.virtual_machine.pk
         kwargs["initial"] = initial
@@ -730,7 +713,14 @@ class IPAddressForm(
         # If `primary_for_parent` is unset, clear the `primary_ip{version}` for the
         # Device/VirtualMachine. It will not be saved until after `IPAddress.clean()` succeeds which
         # also checks for the `_primary_ip_unset_by_form` value.
-        if not primary_for_parent and self.instance._original_assigned_object is not None:
+        device_primary_ip = Device.objects.filter(Q(primary_ip6=self.instance) | Q(primary_ip4=self.instance)).exists()
+        vm_primary_ip = VirtualMachine.objects.filter(
+            Q(primary_ip6=self.instance) | Q(primary_ip4=self.instance)
+        ).exists()
+
+        currently_primary_ip = device_primary_ip or vm_primary_ip
+
+        if not primary_for_parent and self.instance._original_assigned_object is not None and currently_primary_ip:
             self.instance._primary_ip_unset_by_form = True
 
     def save(self, *args, **kwargs):
@@ -759,13 +749,12 @@ class IPAddressBulkCreateForm(BootstrapMixin, forms.Form):
     pattern = ExpandableIPAddressField(label="Address pattern")
 
 
-class IPAddressBulkAddForm(BootstrapMixin, TenancyForm, AddressFieldMixin, CustomFieldModelForm):
+class IPAddressBulkAddForm(NautobotModelForm, TenancyForm, AddressFieldMixin):
     vrf = DynamicModelChoiceField(
         queryset=VRF.objects.all(),
         required=False,
         label="VRF",
     )
-    tags = DynamicModelMultipleChoiceField(queryset=Tag.objects.all(), required=False)
 
     class Meta:
         model = IPAddress
@@ -887,7 +876,7 @@ class IPAddressCSVForm(StatusModelCSVFormMixin, AddressFieldMixin, CustomFieldMo
         return ipaddress
 
 
-class IPAddressBulkEditForm(BootstrapMixin, AddRemoveTagsForm, StatusBulkEditFormMixin, CustomFieldBulkEditForm):
+class IPAddressBulkEditForm(TagsBulkEditFormMixin, StatusModelBulkEditFormMixin, NautobotBulkEditForm):
     pk = forms.ModelMultipleChoiceField(queryset=IPAddress.objects.all(), widget=forms.MultipleHiddenInput())
     vrf = DynamicModelChoiceField(
         queryset=VRF.objects.all(),
@@ -926,7 +915,7 @@ class IPAddressAssignForm(BootstrapMixin, forms.Form):
     )
 
 
-class IPAddressFilterForm(BootstrapMixin, TenancyFilterForm, StatusFilterFormMixin, CustomFieldFilterForm):
+class IPAddressFilterForm(NautobotFilterForm, TenancyFilterForm, StatusModelFilterFormMixin):
     model = IPAddress
     field_order = [
         "q",
@@ -984,13 +973,7 @@ class IPAddressFilterForm(BootstrapMixin, TenancyFilterForm, StatusFilterFormMix
 #
 
 
-class VLANGroupForm(BootstrapMixin, CustomFieldModelForm, RelationshipModelForm):
-    region = DynamicModelChoiceField(queryset=Region.objects.all(), required=False, initial_params={"sites": "$site"})
-    site = DynamicModelChoiceField(
-        queryset=Site.objects.all(),
-        required=False,
-        query_params={"region_id": "$region"},
-    )
+class VLANGroupForm(LocatableModelFormMixin, NautobotModelForm):
     slug = SlugField()
 
     class Meta:
@@ -998,35 +981,21 @@ class VLANGroupForm(BootstrapMixin, CustomFieldModelForm, RelationshipModelForm)
         fields = [
             "region",
             "site",
+            "location",
             "name",
             "slug",
             "description",
         ]
 
 
-class VLANGroupCSVForm(CustomFieldModelCSVForm):
-    site = CSVModelChoiceField(
-        queryset=Site.objects.all(),
-        required=False,
-        to_field_name="name",
-        help_text="Assigned site",
-    )
-
+class VLANGroupCSVForm(LocatableModelCSVFormMixin, CustomFieldModelCSVForm):
     class Meta:
         model = VLANGroup
         fields = VLANGroup.csv_headers
 
 
-class VLANGroupFilterForm(BootstrapMixin, CustomFieldFilterForm):
+class VLANGroupFilterForm(NautobotFilterForm, LocatableModelFilterFormMixin):
     model = VLANGroup
-    region = DynamicModelMultipleChoiceField(queryset=Region.objects.all(), to_field_name="slug", required=False)
-    site = DynamicModelMultipleChoiceField(
-        queryset=Site.objects.all(),
-        to_field_name="slug",
-        required=False,
-        null_option="None",
-        query_params={"region": "$region"},
-    )
 
 
 #
@@ -1034,26 +1003,19 @@ class VLANGroupFilterForm(BootstrapMixin, CustomFieldFilterForm):
 #
 
 
-class VLANForm(BootstrapMixin, TenancyForm, CustomFieldModelForm, RelationshipModelForm):
-    region = DynamicModelChoiceField(queryset=Region.objects.all(), required=False, initial_params={"sites": "$site"})
-    site = DynamicModelChoiceField(
-        queryset=Site.objects.all(),
-        required=False,
-        null_option="None",
-        query_params={"region_id": "$region"},
-    )
+class VLANForm(LocatableModelFormMixin, NautobotModelForm, TenancyForm):
     group = DynamicModelChoiceField(
         queryset=VLANGroup.objects.all(),
         required=False,
         query_params={"site_id": "$site"},
     )
     role = DynamicModelChoiceField(queryset=Role.objects.all(), required=False)
-    tags = DynamicModelMultipleChoiceField(queryset=Tag.objects.all(), required=False)
 
     class Meta:
         model = VLAN
         fields = [
             "site",
+            "location",
             "group",
             "vid",
             "name",
@@ -1074,13 +1036,7 @@ class VLANForm(BootstrapMixin, TenancyForm, CustomFieldModelForm, RelationshipMo
         }
 
 
-class VLANCSVForm(StatusModelCSVFormMixin, CustomFieldModelCSVForm):
-    site = CSVModelChoiceField(
-        queryset=Site.objects.all(),
-        required=False,
-        to_field_name="name",
-        help_text="Assigned site",
-    )
+class VLANCSVForm(LocatableModelCSVFormMixin, StatusModelCSVFormMixin, CustomFieldModelCSVForm):
     group = CSVModelChoiceField(
         queryset=VLANGroup.objects.all(),
         required=False,
@@ -1118,10 +1074,13 @@ class VLANCSVForm(StatusModelCSVFormMixin, CustomFieldModelCSVForm):
             self.fields["group"].queryset = self.fields["group"].queryset.filter(**params)
 
 
-class VLANBulkEditForm(BootstrapMixin, AddRemoveTagsForm, StatusBulkEditFormMixin, CustomFieldBulkEditForm):
+class VLANBulkEditForm(
+    TagsBulkEditFormMixin,
+    LocatableModelBulkEditFormMixin,
+    StatusModelBulkEditFormMixin,
+    NautobotBulkEditForm,
+):
     pk = forms.ModelMultipleChoiceField(queryset=VLAN.objects.all(), widget=forms.MultipleHiddenInput())
-    region = DynamicModelChoiceField(queryset=Region.objects.all(), required=False, to_field_name="slug")
-    site = DynamicModelChoiceField(queryset=Site.objects.all(), required=False, query_params={"region": "$region"})
     group = DynamicModelChoiceField(
         queryset=VLANGroup.objects.all(),
         required=False,
@@ -1132,8 +1091,10 @@ class VLANBulkEditForm(BootstrapMixin, AddRemoveTagsForm, StatusBulkEditFormMixi
     description = forms.CharField(max_length=100, required=False)
 
     class Meta:
+        model = VLAN
         nullable_fields = [
             "site",
+            "location",
             "group",
             "tenant",
             "role",
@@ -1141,12 +1102,13 @@ class VLANBulkEditForm(BootstrapMixin, AddRemoveTagsForm, StatusBulkEditFormMixi
         ]
 
 
-class VLANFilterForm(BootstrapMixin, TenancyFilterForm, StatusFilterFormMixin, CustomFieldFilterForm):
+class VLANFilterForm(NautobotFilterForm, LocatableModelFilterFormMixin, TenancyFilterForm, StatusModelFilterFormMixin):
     model = VLAN
     field_order = [
         "q",
         "region",
         "site",
+        "location",
         "group_id",
         "status",
         "role",
@@ -1154,14 +1116,6 @@ class VLANFilterForm(BootstrapMixin, TenancyFilterForm, StatusFilterFormMixin, C
         "tenant",
     ]
     q = forms.CharField(required=False, label="Search")
-    region = DynamicModelMultipleChoiceField(queryset=Region.objects.all(), to_field_name="slug", required=False)
-    site = DynamicModelMultipleChoiceField(
-        queryset=Site.objects.all(),
-        to_field_name="slug",
-        required=False,
-        null_option="None",
-        query_params={"region": "$region"},
-    )
     group_id = DynamicModelMultipleChoiceField(
         queryset=VLANGroup.objects.all(),
         required=False,
@@ -1183,12 +1137,11 @@ class VLANFilterForm(BootstrapMixin, TenancyFilterForm, StatusFilterFormMixin, C
 #
 
 
-class ServiceForm(BootstrapMixin, CustomFieldModelForm, RelationshipModelForm):
+class ServiceForm(NautobotModelForm):
     ports = NumericArrayField(
         base_field=forms.IntegerField(min_value=SERVICE_PORT_MIN, max_value=SERVICE_PORT_MAX),
         help_text="Comma-separated list of one or more port numbers. A range may be specified using a hyphen.",
     )
-    tags = DynamicModelMultipleChoiceField(queryset=Tag.objects.all(), required=False)
 
     class Meta:
         model = Service
@@ -1225,7 +1178,7 @@ class ServiceForm(BootstrapMixin, CustomFieldModelForm, RelationshipModelForm):
             self.fields["ipaddresses"].choices = []
 
 
-class ServiceFilterForm(BootstrapMixin, CustomFieldFilterForm):
+class ServiceFilterForm(NautobotFilterForm):
     model = Service
     q = forms.CharField(required=False, label="Search")
     protocol = forms.ChoiceField(
@@ -1259,7 +1212,7 @@ class ServiceCSVForm(CustomFieldModelCSVForm):
         fields = Service.csv_headers
 
 
-class ServiceBulkEditForm(BootstrapMixin, AddRemoveTagsForm, CustomFieldBulkEditForm):
+class ServiceBulkEditForm(TagsBulkEditFormMixin, NautobotBulkEditForm):
     pk = forms.ModelMultipleChoiceField(queryset=Service.objects.all(), widget=forms.MultipleHiddenInput())
     protocol = forms.ChoiceField(
         choices=add_blank_choice(ServiceProtocolChoices),
