@@ -1,7 +1,6 @@
 from unittest import skipIf
 
 import netaddr
-from django.test import TestCase
 from django.db import connection
 
 
@@ -28,8 +27,6 @@ from nautobot.ipam.filters import (
     VLANGroupFilterSet,
     VRFFilterSet,
 )
-
-
 from nautobot.ipam.models import (
     Aggregate,
     IPAddress,
@@ -42,16 +39,17 @@ from nautobot.ipam.models import (
     VLANGroup,
     VRF,
 )
+from nautobot.tenancy.models import Tenant, TenantGroup
+from nautobot.utilities.testing import TestCase, FilterTestCases
 from nautobot.virtualization.models import (
     Cluster,
     ClusterType,
     VirtualMachine,
     VMInterface,
 )
-from nautobot.tenancy.models import Tenant, TenantGroup
 
 
-class VRFTestCase(TestCase):
+class VRFTestCase(FilterTestCases.FilterTestCase):
     queryset = VRF.objects.all()
     filterset = VRFFilterSet
 
@@ -91,10 +89,6 @@ class VRFTestCase(TestCase):
         vrfs[2].import_targets.add(route_targets[2])
         vrfs[2].export_targets.add(route_targets[2])
 
-    def test_id(self):
-        params = {"id": self.queryset.values_list("pk", flat=True)[:2]}
-        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 2)
-
     def test_name(self):
         params = {"name": ["VRF 1", "VRF 2"]}
         self.assertEqual(self.filterset(params, self.queryset).qs.count(), 2)
@@ -143,7 +137,7 @@ class VRFTestCase(TestCase):
         self.assertEqual(self.filterset(params, self.queryset).qs.values_list("pk", flat=True)[0], value)
 
 
-class RouteTargetTestCase(TestCase):
+class RouteTargetTestCase(FilterTestCases.FilterTestCase):
     queryset = RouteTarget.objects.all()
     filterset = RouteTargetFilterSet
 
@@ -187,10 +181,6 @@ class RouteTargetTestCase(TestCase):
         vrfs[1].import_targets.add(route_targets[4], route_targets[5])
         vrfs[1].export_targets.add(route_targets[6], route_targets[7])
 
-    def test_id(self):
-        params = {"id": self.queryset.values_list("pk", flat=True)[:2]}
-        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 2)
-
     def test_name(self):
         params = {"name": ["65000:1001", "65000:1002", "65000:1003"]}
         self.assertEqual(self.filterset(params, self.queryset).qs.count(), 3)
@@ -229,7 +219,7 @@ class RouteTargetTestCase(TestCase):
         self.assertEqual(self.filterset(params, self.queryset).qs.values_list("pk", flat=True)[0], value)
 
 
-class RIRTestCase(TestCase):
+class RIRTestCase(FilterTestCases.NameSlugFilterTestCase):
     queryset = RIR.objects.all()
     filterset = RIRFilterSet
 
@@ -243,18 +233,6 @@ class RIRTestCase(TestCase):
         RIR.objects.create(name="RIR 5", slug="rir-5", is_private=True, description="E"),
         RIR.objects.create(name="RIR 6", slug="rir-6", is_private=True, description="F"),
 
-    def test_id(self):
-        params = {"id": self.queryset.values_list("pk", flat=True)[:2]}
-        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 2)
-
-    def test_name(self):
-        params = {"name": ["RIR 1", "RIR 2"]}
-        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 2)
-
-    def test_slug(self):
-        params = {"slug": ["rir-1", "rir-2"]}
-        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 2)
-
     def test_description(self):
         params = {"description": ["A", "B"]}
         self.assertEqual(self.filterset(params, self.queryset).qs.count(), 2)
@@ -266,7 +244,7 @@ class RIRTestCase(TestCase):
         self.assertEqual(self.filterset(params, self.queryset).qs.count(), 3)
 
 
-class AggregateTestCase(TestCase):
+class AggregateTestCase(FilterTestCases.FilterTestCase):
     queryset = Aggregate.objects.all()
     filterset = AggregateFilterSet
 
@@ -341,10 +319,6 @@ class AggregateTestCase(TestCase):
             params = {"q": value}
             self.assertEqual(self.filterset(params, self.queryset).qs.count(), 1)
 
-    def test_id(self):
-        params = {"id": self.queryset.values_list("pk", flat=True)[:2]}
-        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 2)
-
     def test_family(self):
         params = {"family": "4"}
         self.assertEqual(self.filterset(params, self.queryset).qs.count(), 3)
@@ -380,7 +354,7 @@ class AggregateTestCase(TestCase):
         self.assertEqual(self.filterset(params, self.queryset).qs.count(), 4)
 
 
-class RoleTestCase(TestCase):
+class RoleTestCase(FilterTestCases.NameSlugFilterTestCase):
     queryset = Role.objects.all()
     filterset = RoleFilterSet
 
@@ -391,20 +365,8 @@ class RoleTestCase(TestCase):
         Role.objects.create(name="Role 2", slug="role-2"),
         Role.objects.create(name="Role 3", slug="role-3"),
 
-    def test_id(self):
-        params = {"id": self.queryset.values_list("pk", flat=True)[:2]}
-        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 2)
 
-    def test_name(self):
-        params = {"name": ["Role 1", "Role 2"]}
-        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 2)
-
-    def test_slug(self):
-        params = {"slug": ["role-1", "role-2"]}
-        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 2)
-
-
-class PrefixTestCase(TestCase):
+class PrefixTestCase(FilterTestCases.FilterTestCase):
     queryset = Prefix.objects.all()
     filterset = PrefixFilterSet
 
@@ -554,10 +516,6 @@ class PrefixTestCase(TestCase):
             params = {"q": value}
             self.assertEqual(self.filterset(params, self.queryset).qs.count(), 1)
 
-    def test_id(self):
-        params = {"id": self.queryset.values_list("pk", flat=True)[:2]}
-        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 2)
-
     def test_family(self):
         params = {"family": "6"}
         self.assertEqual(self.filterset(params, self.queryset).qs.count(), 5)
@@ -649,7 +607,7 @@ class PrefixTestCase(TestCase):
         self.assertEqual(self.filterset(params, self.queryset).qs.count(), 4)
 
 
-class IPAddressTestCase(TestCase):
+class IPAddressTestCase(FilterTestCases.FilterTestCase):
     queryset = IPAddress.objects.all()
     filterset = IPAddressFilterSet
 
@@ -840,10 +798,6 @@ class IPAddressTestCase(TestCase):
             params = {"q": term}
             self.assertEqual(self.filterset(params, self.queryset).qs.count(), cnt)
 
-    def test_id(self):
-        params = {"id": self.queryset.values_list("pk", flat=True)[:2]}
-        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 2)
-
     def test_family(self):
         params = {"family": "6"}
         self.assertEqual(self.filterset(params, self.queryset).qs.count(), 5)
@@ -941,7 +895,7 @@ class IPAddressTestCase(TestCase):
         self.assertEqual(self.filterset(params, self.queryset).qs.count(), 4)
 
 
-class VLANGroupTestCase(TestCase):
+class VLANGroupTestCase(FilterTestCases.NameSlugFilterTestCase):
     queryset = VLANGroup.objects.all()
     filterset = VLANGroupFilterSet
 
@@ -965,18 +919,6 @@ class VLANGroupTestCase(TestCase):
         VLANGroup.objects.create(name="VLAN Group 3", slug="vlan-group-3", site=sites[2], description="C"),
         VLANGroup.objects.create(name="VLAN Group 4", slug="vlan-group-4", site=None),
 
-    def test_id(self):
-        params = {"id": self.queryset.values_list("pk", flat=True)[:2]}
-        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 2)
-
-    def test_name(self):
-        params = {"name": ["VLAN Group 1", "VLAN Group 2"]}
-        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 2)
-
-    def test_slug(self):
-        params = {"slug": ["vlan-group-1", "vlan-group-2"]}
-        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 2)
-
     def test_description(self):
         params = {"description": ["A", "B"]}
         self.assertEqual(self.filterset(params, self.queryset).qs.count(), 2)
@@ -996,7 +938,7 @@ class VLANGroupTestCase(TestCase):
         self.assertEqual(self.filterset(params, self.queryset).qs.count(), 2)
 
 
-class VLANTestCase(TestCase):
+class VLANTestCase(FilterTestCases.FilterTestCase):
     queryset = VLAN.objects.all()
     filterset = VLANFilterSet
 
@@ -1097,10 +1039,6 @@ class VLANTestCase(TestCase):
             status=status_map["reserved"],
         )
 
-    def test_id(self):
-        params = {"id": self.queryset.values_list("pk", flat=True)[:2]}
-        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 2)
-
     def test_name(self):
         params = {"name": ["VLAN 101", "VLAN 102"]}
         self.assertEqual(self.filterset(params, self.queryset).qs.count(), 2)
@@ -1160,8 +1098,17 @@ class VLANTestCase(TestCase):
         params = {"q": value}
         self.assertEqual(self.filterset(params, self.queryset).qs.values_list("pk", flat=True)[0], value)
 
+    def test_available_on_device(self):
+        manufacturer = Manufacturer.objects.create(name="Test Manufacturer 1", slug="test-manufacturer-1")
+        devicetype = DeviceType.objects.create(manufacturer=manufacturer, model="Device Type 1", slug="device-type-1")
+        site = Site.objects.get(slug="test-site-1")
+        devicerole = DeviceRole.objects.create(name="Test Device Role 1", slug="test-device-role-1", color="ff0000")
+        device = Device.objects.create(device_type=devicetype, device_role=devicerole, name="Device 1", site=site)
+        params = {"available_on_device": device.pk}
+        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 2)
 
-class ServiceTestCase(TestCase):
+
+class ServiceTestCase(FilterTestCases.FilterTestCase):
     queryset = Service.objects.all()
     filterset = ServiceFilterSet
 
@@ -1240,10 +1187,6 @@ class ServiceTestCase(TestCase):
             ports=[2003],
         ),
 
-    def test_id(self):
-        params = {"id": self.queryset.values_list("pk", flat=True)[:3]}
-        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 3)
-
     def test_name(self):
         params = {"name": ["Service 1", "Service 2"]}
         self.assertEqual(self.filterset(params, self.queryset).qs.count(), 2)
