@@ -1,16 +1,13 @@
 from django import forms
 
 from nautobot.extras.forms import (
-    AddRemoveTagsForm,
-    CustomFieldModelForm,
-    RelationshipModelForm,
-    CustomFieldBulkEditForm,
-    CustomFieldFilterForm,
+    NautobotFilterForm,
     CustomFieldModelCSVForm,
+    NautobotBulkEditForm,
+    NautobotModelForm,
+    TagsBulkEditFormMixin,
 )
-from nautobot.extras.models import Tag
 from nautobot.utilities.forms import (
-    BootstrapMixin,
     CommentField,
     CSVModelChoiceField,
     DynamicModelChoiceField,
@@ -26,7 +23,7 @@ from .models import Tenant, TenantGroup
 #
 
 
-class TenantGroupForm(BootstrapMixin, CustomFieldModelForm, RelationshipModelForm):
+class TenantGroupForm(NautobotModelForm):
     parent = DynamicModelChoiceField(queryset=TenantGroup.objects.all(), required=False)
     slug = SlugField()
 
@@ -58,11 +55,10 @@ class TenantGroupCSVForm(CustomFieldModelCSVForm):
 #
 
 
-class TenantForm(BootstrapMixin, CustomFieldModelForm, RelationshipModelForm):
+class TenantForm(NautobotModelForm):
     slug = SlugField()
     group = DynamicModelChoiceField(queryset=TenantGroup.objects.all(), required=False)
     comments = CommentField()
-    tags = DynamicModelMultipleChoiceField(queryset=Tag.objects.all(), required=False)
 
     class Meta:
         model = Tenant
@@ -89,7 +85,7 @@ class TenantCSVForm(CustomFieldModelCSVForm):
         fields = Tenant.csv_headers
 
 
-class TenantBulkEditForm(BootstrapMixin, AddRemoveTagsForm, CustomFieldBulkEditForm):
+class TenantBulkEditForm(TagsBulkEditFormMixin, NautobotBulkEditForm):
     pk = forms.ModelMultipleChoiceField(queryset=Tenant.objects.all(), widget=forms.MultipleHiddenInput())
     group = DynamicModelChoiceField(queryset=TenantGroup.objects.all(), required=False)
 
@@ -99,7 +95,7 @@ class TenantBulkEditForm(BootstrapMixin, AddRemoveTagsForm, CustomFieldBulkEditF
         ]
 
 
-class TenantFilterForm(BootstrapMixin, CustomFieldFilterForm):
+class TenantFilterForm(NautobotFilterForm):
     model = Tenant
     q = forms.CharField(required=False, label="Search")
     group = DynamicModelMultipleChoiceField(
