@@ -1,6 +1,8 @@
 from django.contrib.auth import get_user_model
 from django.contrib.auth.models import Group
 from django.db.models import Count
+from drf_spectacular.utils import extend_schema, OpenApiTypes
+from rest_framework.authentication import BasicAuthentication
 from rest_framework.permissions import IsAuthenticated
 from rest_framework.response import Response
 from rest_framework.routers import APIRootView
@@ -8,7 +10,7 @@ from rest_framework.viewsets import ViewSet
 
 from nautobot.core.api.views import ModelViewSet
 from nautobot.users import filters
-from nautobot.users.models import ObjectPermission
+from nautobot.users.models import ObjectPermission, Token
 from nautobot.utilities.querysets import RestrictedQuerySet
 from nautobot.utilities.utils import deepmerge
 from . import serializers
@@ -40,6 +42,30 @@ class GroupViewSet(ModelViewSet):
     filterset_class = filters.GroupFilterSet
 
 
+#
+# REST API tokens
+#
+
+
+class TokenViewSet(ModelViewSet):
+    queryset = RestrictedQuerySet(model=Token).prefetch_related("user")
+    serializer_class = serializers.TokenSerializer
+    filterset_class = filters.TokenFilterSet
+
+    @property
+    def authentication_classes(self):
+        """Inherit default authentication_classes and basic authentication."""
+        classes = super().authentication_classes
+        return classes + [BasicAuthentication]
+
+    def get_queryset(self):
+        """
+        Limit users to their own Tokens.
+        """
+        queryset = super().get_queryset()
+        return queryset.filter(user=self.request.user)
+
+
 #
 # ObjectPermissions
 #
@@ -63,12 +89,14 @@ class UserConfigViewSet(ViewSet):
 
     permission_classes = [IsAuthenticated]
 
+    @extend_schema(responses={200: OpenApiTypes.OBJECT})
     def list(self, request):
         """
         Return the config_data for the currently authenticated User.
         """
         return Response(request.user.config_data)
 
+    @extend_schema(request=OpenApiTypes.OBJECT)
     def patch(self, request):
         """
         Update the config_data for the currently authenticated User.
