@@ -1,5 +1,6 @@
 """General-purpose Git utilities."""
 
+from collections import namedtuple
 import logging
 import os
 
@@ -8,20 +9,63 @@ from git import Repo
 
 logger = logging.getLogger("nautobot.utilities.git")
 
+# namedtuple takes a git log diff status and its accompanying text.
+GitDiffLog = namedtuple("GitDiffLog", ["status", "text"])
+
+# 'A' and 'D' status are swapped because of the way the repo.git.diff was implemented
+# e.g. 'A' actually stands for Addition but in this case is Deletion
+GIT_STATUS_MAP = {
+    "A": "Deletion",
+    "M": "Modification",
+    "C": "Copy",
+    "D": "Addition",
+    "R": "Renaming",
+    "T": "File Type Changed",
+    "U": "File Unmerged",
+    "X": "Unknown",
+}
+
+
+def swap_status_initials(data):
+    """Swap Git status initials with its equivalent."""
+    initial, text = data.split("\t")
+    return GitDiffLog(status=GIT_STATUS_MAP.get(initial), text=text)
+
+
+def convert_git_diff_log_to_list(logs):
+    """
+    Convert Git diff log into a list splitted by \\n
+
+    Example:
+        >>> git_log = "M\tindex.html\nR\tsample.txt"
+        >>> print(convert_git_diff_log_to_list(git_log))
+        ["Modification - index.html", "Renaming - sample.txt"]
+    """
+    logs = logs.split("\n")
+    return [swap_status_initials(line) for line in logs]
+
 
 class BranchDoesNotExist(Exception):
     pass
 
 
 class GitRepo:
-    def __init__(self, path, url):
+    def __init__(self, path, url, clone_initially=True):
         """
         Ensure that we have a clone of the given remote Git repository URL at the given local directory path.
+
+        Args:
+            path (str): path to git repo
+            url (str): git repo url
+            clone_initially (bool): True if the repo needs to be cloned
         """
         if os.path.isdir(path):
             self.repo = Repo(path=path)
-        else:
+        elif clone_initially:
             self.repo = Repo.clone_from(url, to_path=path)
+        else:
+            self.repo = Repo.init(path)
+            self.repo.create_remote("origin", url=url)
 
         if url not in self.repo.remotes.origin.urls:
             self.repo.remotes.origin.set_url(url)
@@ -75,3 +119,25 @@ class GitRepo:
         commit_hexsha = self.repo.head.reference.commit.hexsha
         logger.info(f"Latest commit on branch `{branch}` is `{commit_hexsha}`")
         return commit_hexsha
+
+    def diff_remote(self, branch):
+        logger.debug("Fetching from remote.")
+        self.fetch()
+
+        try:
+            self.repo.remotes.origin.refs[branch]
+        except IndexError as git_error:
+            logger.error(
+                "Branch %s does not exist at %s. %s", branch, list(self.repo.remotes.origin.urls)[0], git_error
+            )
+            raise BranchDoesNotExist(
+                f"Please create branch '{branch}' in upstream and try again."
+                f" If this is a new repo, please add a commit before syncing. {git_error}"
+            )
+
+        logger.debug("Getting diff between local branch and remote branch")
+        diff = self.repo.git.diff("--name-status", f"origin/{branch}")
+        if diff:  # if diff is not empty
+            return convert_git_diff_log_to_list(diff)
+        logger.debug("No Difference")
+        return []
