@@ -12,9 +12,11 @@ from django.utils.text import Truncator
 from django_tables2.data import TableQuerysetData
 from django_tables2.utils import Accessor
 
-from nautobot.extras.models import CustomField
+from nautobot.extras.models import ComputedField, CustomField
 from nautobot.extras.choices import CustomFieldTypeChoices
 
+from .templatetags.helpers import render_boolean
+
 
 class BaseTable(tables.Table):
     """
@@ -33,9 +35,12 @@ class BaseTable(tables.Table):
         obj_type = ContentType.objects.get_for_model(self._meta.model)
 
         for cf in CustomField.objects.filter(content_types=obj_type):
-            name = "cf_{}".format(cf.name)
+            name = "cf_{}".format(cf.slug)
             self.base_columns[name] = CustomFieldColumn(cf)
 
+        for cpf in ComputedField.objects.filter(content_type=obj_type):
+            self.base_columns[f"cpf_{cpf.slug}"] = ComputedFieldColumn(cpf)
+
         # Init table
         super().__init__(*args, **kwargs)
 
@@ -145,13 +150,7 @@ class BooleanColumn(tables.Column):
     """
 
     def render(self, value):
-        if value:
-            rendered = '<span class="text-success"><i class="mdi mdi-check-bold"></i></span>'
-        elif value is None:
-            rendered = '<span class="text-muted">&mdash;</span>'
-        else:
-            rendered = '<span class="text-danger"><i class="mdi mdi-close-thick"></i></span>'
-        return mark_safe(rendered)
+        return render_boolean(value)
 
 
 class ButtonsColumn(tables.TemplateColumn):
@@ -335,6 +334,21 @@ class ContentTypesColumn(tables.ManyToManyColumn):
         return value
 
 
+class ComputedFieldColumn(tables.Column):
+    """
+    Display computed fields in the appropriate format.
+    """
+
+    def __init__(self, computedfield, *args, **kwargs):
+        self.computedfield = computedfield
+        kwargs["verbose_name"] = computedfield.label
+
+        super().__init__(*args, empty_values=[], **kwargs)
+
+    def render(self, record):
+        return self.computedfield.render({"obj": record})
+
+
 class CustomFieldColumn(tables.Column):
     """
     Display custom fields in the appropriate format.
@@ -342,6 +356,7 @@ class CustomFieldColumn(tables.Column):
 
     def __init__(self, customfield, *args, **kwargs):
         self.customfield = customfield
+        # 2.0 TODO: #824 replace customfield.name with customfield.slug
         kwargs["accessor"] = Accessor(f"_custom_field_data__{customfield.name}")
         kwargs["verbose_name"] = customfield.label or customfield.name
 
@@ -351,16 +366,14 @@ class CustomFieldColumn(tables.Column):
         if value is None:
             return self.default
 
+        template = ""
         if self.customfield.type == CustomFieldTypeChoices.TYPE_BOOLEAN:
-            if value is True:
-                template = '<span class="text-success"><i class="mdi mdi-check-bold"></i></span>'
-            else:
-                template = '<span class="text-danger"><i class="mdi mdi-close-thick"></i></span>'
+            template = render_boolean(value)
         elif self.customfield.type == CustomFieldTypeChoices.TYPE_MULTISELECT:
-            if value:
-                template = ""
-                for v in value:
-                    template += format_html('<span class="label label-default">{}</span> ', v)
+            if not value:
+                return self.default
+            for v in value:
+                template += format_html('<span class="label label-default">{}</span> ', v)
         elif self.customfield.type == CustomFieldTypeChoices.TYPE_SELECT:
             template = format_html('<span class="label label-default">{}</span>', value)
         elif self.customfield.type == CustomFieldTypeChoices.TYPE_URL:
